#pragma rtGlobals=1		// Use modern global access method.
// Last Mod: 6/26

//////////////////////////////////////
// Makes globals so camera handle can be accessed by other functions
//////////////////////////////////////
Function MakeCameraGlobals()

	String dfSav = GetDataFolder(1)

	If(DataFolderExists("root:Globals:imaging:CameraHandleGlobals"))
		return 0
	Else
		NewDataFolder/O/S root:Globals:imaging:CameraHandleGlobals
		Variable/G GCamera_handle
		Variable/G GSIDX_handle
		Variable/G CameraHandleExists
		Variable/G SIDXHandleExists
		Variable/G totalROIs
		
	EndIf
	
	SetDataFolder dfSav
End

//////////////////////////////////////
// Makes globals that control basic aspects of image aquisition (binning, exposure, frames, etc.)
//////////////////////////////////////
Function MakeCameraParameterGlobals()
	String dfSav = GetDataFolder(1)

	If(DataFolderExists("root:Globals:imaging:CameraParameterGlobals"))
		return 0
	Else
		NewDataFolder/O/S root:Globals:imaging:CameraParameterGlobals
		
		Variable/G X_pixels=2
		Variable/G Y_pixels=2
		Variable/G exposure_time=.06
		Variable/G num_frames=10
		//Variable/G IlluminationChannel= -1	// Which DAC or TTL interfaces w/ the light source? "-1" indicates not initialized yet
		//Variable/G TillLampPresent	// Using monochromator?
		//Variable/G TillLamp_Lambda 	// Wavelength to deliver? 
		
				
	EndIf
	
	SetDataFolder dfSav
End

//////////////////////////////////////
// Sets up a handle for the camera and stores it in a global. Also creates flags
// to indicate whether handles exist, are corrupted, etc..
//////////////////////////////////////
Function InitializeCamera()
	MakeCameraGlobals()

	//GLOBALS //
	NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
	NVAR sidx_handle=root:Globals:imaging:CameraHandleGlobals:GSIDX_handle
	NVAR camera_HasHandle=root:Globals:imaging:CameraHandleGlobals:CameraHandleExists
	NVAR sidx_HasHandle=root:Globals:imaging:CameraHandleGlobals:SIDXHandleExists
	//NVAR IlluminationChannel = root:Globals:imaging:CameraHandleGlobals:IlluminationChannel
	//----------------//

	If(sidx_HasHandle)
		DoAlert 0, "The camera is already initialized"
		return 0
	EndIf


	Variable status,canceled
	String message

	SIDXOpen sidx_handle,status	
	
	If(status!=0)
		SIDXGetStatusText sidx_handle,status,message
		printf "%s:%s","SIDXOpen",message
		return 0
	EndIf

			
	SIDXDialogCameraSelector sidx_handle,canceled,camera_handle,status

	If(status!=0)
		SIDXGetStatusText sidx_handle,status,message
		DeallocateCamera()
		printf "%s: %s", "SIDXDialogCameraSelector", message
		return 0
	Endif
	
	If(canceled!=0)
		DeallocateCamera()
		print "Camera dialog canceled"
		return 0
	EndIf

	SIDXDialogCamera camera_handle,canceled,status

	If(status!=0)
		SIDXCameraGetStatusText camera_handle,status,message
		DeallocateCamera()
		printf "%s: %s", "SIDXDialogCamera", message
		return 0
	EndIf

	If (canceled != 0)
		DeallocateCamera()
		print "Camera dialog canceled."
		return 0
	Endif

	sidx_HasHandle=1
	camera_HasHandle=1

End


//////////////////////////////////////
// This basically just closes the camera and SIDX, but also kills the data folder w/ the handles.
// This is useful for keeping track of whether a global for the camera handle exists.
//////////////////////////////////////
Function DeallocateCamera()

	If(!DataFolderExists("root:Globals:imaging:CameraHandleGlobals"))
		return 0
	EndIf
	
	//GLOBALS //
	NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
	NVAR sidx_handle=root:Globals:imaging:CameraHandleGlobals:GSIDX_handle
	//----------------//

	SIDXCameraClose camera_handle
	SIDXClose sidx_handle

	KillDataFolder/Z root:Globals:imaging:CameraHandleGlobals

End
//////////////////////////////////////
// Updates camera settings(Binning, exposure). More advanced settings (shutter mode,
// etc.) are camera specific and should be taken care of by other fxns.
//////////////////////////////////////
Function CameraParameters()

	//GLOBALS //
	NVAR camera_HasHandle=root:Globals:imaging:CameraHandleGlobals:CameraHandleExists
	NVAR X_pixels=root:Globals:imaging:CameraParameterGlobals:X_Pixels
	NVAR Y_pixels=root:Globals:imaging:CameraParameterGlobals:Y_Pixels
	NVAR exposure_time=root:Globals:imaging:CameraParameterGlobals:exposure_time
	NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
	NVAR sidx_handle=root:Globals:imaging:CameraHandleGlobals:GSIDX_handle
	//----------------//
	
	Variable exposure_get,status
	
	If(!camera_HasHandle || !DataFolderExists("root:Globals:imaging:CameraHandleGlobals"))
		DoAlert 0, "Camera needs to be initialized before calling CameraParameters()"
		Abort
	EndIf
	
	
	SIDXExposureTimeSet camera_handle,exposure_time,status
	SIDXBinningSet camera_handle,X_pixels,Y_pixels,status	
		
End

//////////////////////////////////////
Function fastFocus()
//////////////////////////////////////	
	//GLOBALS //
	NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
	NVAR sidx_handle=root:Globals:imaging:CameraHandleGlobals:GSIDX_handle
	NVAR TillLampPresent = root:Globals:physiology:Run_globals:TillLampPresent ///////////// THIS NEEDS TO GO.... ///////////////
	NVAR IlluminationChannel = root:Globals:physiology:Run_Globals:IlluminationChannel /////////////// ... THIS TOO... ///////////////
	NVAR CameraHasHandle = root:Globals:imaging:CameraHandleGlobals:CameraHandleExists
	//Wave FrameToShow = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow
	//----------------//
	
	setDataFolder root:
	
	If(!DataFolderExists("root:Globals:imaging:CameraHandleGlobals") || !CameraHasHandle)
		DoAlert 0, "Camera needs to be initialized first"
		return 0
	EndIf
	

	cameraParameters()
	
	Variable status
	
	String message,WaveImage
	WaveImage="waveImage"
	
//	SIDXAcquisitionStop camera_handle
	SIDXTriggerSet camera_handle,1,status	
	SIDXAcquisitionBeginFocus camera_handle, status // Prepare the camera for focus mode
	

	Variable frames,roi_index,utc_time
	Variable bytes, index, x_pixels, y_pixels
	Variable locked, hardware_provided, temperature

	LightOn()
	
	SIDXAcquisitionStart camera_handle, status // Start the actual acquisition. 

	
	roi_index = 0

	Variable V_abortcode
	Variable firstFrame=1
	
	try	// focus will continue indefinitely until it is either aborted, or user ESC's out
		
		Do
			
			SIDXAcquisitionFramesAvailable camera_handle, frames, status 
		
			If (frames > 0)	// Is a frame ready?
				SIDXAcquisitionGetFrames camera_handle, frames, roi_index, $waveImage, utc_time, status // ...Write input to a single IGOR wave
				
				Duplicate/O $waveImage root:globals:Imaging:ImageAnalysisGlobals:frameToShow
			
			// Need to make this able to write to an arbitrary window, not just the frame viewer....
				
				If(FirstFrame)
					ControlInfo/W=FrameViewer_subwin#ImageControls ContrastPicker
					Switch (V_value)
					
					case 1:
						LinearContrast()
						break
					case 2:
						InvertContrast()
						break
					case 3:
						GammaContrast()
						break
					default:
						break
						
					EndSwitch
					
					FirstFrame+=1
				EndIf
				
				GetFocusState()	// Adjust contrast,brightness, etc. 
			Endif
			

			Variable pressed = GetKeyState(0)	// detect user presses inside the main Do-loop
			
			If(pressed==32)	// Did the user hit escape?
				terminateAcquisition()
				LightOff()
				break
			EndIf
		
		While(1)	// loop until user breaks 

	catch
		
		If(V_abortCode==1) // 
			TerminateAcquisition()
			LightOff()
		EndIf
		
	endtry

	

End



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Similar motivation to "deallocate camera" Just used to make the command folder+global
// saavy.
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function TerminateAcquisition()

	//GLOBALS //
	NVAR/Z camera_handle = root:Globals:imaging:CameraHandleGlobals:GCamera_handle 
	//----------------//
	
	Variable status,bytes,x_pixels,y_pixels
	Variable roi_index=0
	Variable temperature,locked,hardware_provided
	
	If(!DataFolderExists("root:Globals:imaging:CameraHandleGlobals") || camera_handle==0)
		return 0
	EndIf


	Print "Terminating Acquisition"
	
	SIDXAcquisitionStop camera_handle
	
	SIDXAcquisitionGetSize camera_handle, 0, bytes, x_pixels, y_pixels, status
	printf "ROI: %d, Bytes: %d, X pixels: %d, Y pixels: %d\r", roi_index, bytes, x_pixels, y_pixels
	SIDXTemperatureGet camera_handle, temperature, locked, hardware_provided, status
	printf "CCD temperature measured: %d, Temperature locked (1 means true): %d, Temperature locked info provided by hardware (1 means true): %d\r", temperature, locked, hardware_provided
	
	SIDXAcquisitionEnd camera_handle

End


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Acquires a single sweep, user specifies total frames and image name.
// This is imaging specific.... not to be confused w/ AcquireOneSweep which collects 
// both phys and imaging... Also need some escape code in appropriate places?? (So camera doesn't 
// end up locked..)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function AcquireOneSweepz(frames,wave_image,totalROIs)
	
	Variable frames
	String wave_image
	Variable totalROIs
	
	
	//GLOBALS //
	NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
	NVAR sidx_handle=root:Globals:imaging:CameraHandleGlobals:GSIDX_handle
	SVAR ExperimentName = root:Globals:physiology:Run_globals:ExperimentName
	SVAR GroupName=root:Globals:physiology:Run_globals:GroupName
	
	NVAR num_frames = root:Globals:imaging:CameraParameterGlobals:num_frames
	NVAR BinX = root:Globals:imaging:CameraParameterGlobals:X_pixels
	NVAR BinY = root:Globals:imaging:CameraParameterGlobals:Y_pixels
	NVAR exposure_time = root:Globals:imaging:CameraParameterGlobals:exposure_time
	
	//----------------//
	
	If(!DataFolderExists("root:Globals:imaging:CameraHandleGlobals"))
		DoAlert 0, "you need to initialize first"
		return 0
	EndIf

	variable status
	
	cameraParameters()	// update camera parameters (frames,bin,etc..)
	
	String message
	
	Variable roi_index,utc_time
	
	SIDXTriggerSet camera_handle,1,status	// no trigger necessary for collecting one sweep.
	SIDXAcquisitionBeginSequence camera_handle, frames,status // prep the camera for acquisition

	If(status!=0)
		SIDXCameraGetStatusText camera_handle,status,message
		DeallocateCamera()
		printf"%s:%s","SIDXAcquisitionBeginSequence",message
		return 0
	EndIf

	SIDXAcquisitionStart camera_handle, status // let er' rip.
	
	String fullWavePath = "root:"+ExperimentName+":imaging_data:"
	
	WriteAvailableImages(wave_image,fullWavePath)	// try this instead of the below for ROIs
		Wave/T imageList = $"root:"+ExperimentName+":ImageList"
		RemoveFromWavelist(ImageList,GroupName)
		imageList[dimsize(imageList,0)-1][0]=GroupName
		imageList[dimsize(imageList,0)-1][1] = "Frames:"+ num2str(num_frames) + ";BinX:"+num2str(BinX)+";BinY:"+num2str(BinY)+";exposure time:"+num2str(exposure_time)+";Collected at:"+time()
		
		Print "root:"+ExperimentName+":ImageList"
		Redimension/N=(dimsize(imageList,0)+1,2) imageList	
		
		
	TerminateAcquisition()
	
	////////////////// NEED TO CLEAN UP THIS SNIPPETT /////////////////////////////
	
	String FrmVwr=WinList("FrameViewer*",";","WIN:1")
		
	If(Strlen(FrmVwr)!=0)		
		MyCheckProc("All",1) // Make sure list of available images is updated for the frame viewer
	EndIf
			
End


/////////////// Clears all available ROIS
Function ClearROIs()

//GLOBALS //
NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
NVAR CameraHandleExists =root:Globals:imaging:CameraHandleGlobals:CameraHandleExists
//----------------//

	If((!DataFolderExists("root:Globals:imaging:CameraHandleGlobals")) || !(CameraHandleExists))
		return 0
	EndIf

Variable index,status
String message
	
	SIDXROIClear camera_handle,status
	
	If(status!=0)
		SIDXCameraGetStatusText camera_handle,status,message
		printf "%s:%s","SIDXROIClear",message
		return 0
	EndIf
	

End

/////// If frames are in the buffer, then write them to to an IGOR wave ////
Function WriteAvailableImages(wave_image, destFolder)
	String wave_Image
	String destFolder
	
	//GLOBALS //
	NVAR camera_handle=root:Globals:imaging:CameraHandleGlobals:GCamera_handle
	NVAR totalROIs = root:Globals:imaging:CameraHandleGlobal:totalROIs
	NVAR frames = root:Globals:imaging:CameraParameterGlobals:num_frames
	SVAR ExperimentName = root:Globals:physiology:Run_globals:ExperimentName	
	//----------------//

	Variable done_count,status
	Variable roi_index,utc_time
	String message

	/// If there's no handle, then need to esc out... // 

	Do
		SIDXAcquisitionFramesAvailable camera_handle,done_count,status	// wait until the frames are ready to be read into a wave..
		
		If(status!=0)
			TerminateAcquisition()
			return 0
		EndIf
		
	While(!done_count)

	SIDXAcquisitionStop camera_handle	// kill the acquisition
	
	String writeWave=destFolder + wave_image
	
	If (DataFolderExists("root:"+ExperimentName + ":imaging_data:"))
		SIDXAcquisitionGetFrames camera_handle,frames,roi_index,$writeWave,utc_time,status	//.. and write to an IGOR wave
	EndIf
	
	If(status!=0)
		SIDXCameraGetStatusText camera_handle,status,message
		TerminateAcquisition()
		printf "%s:%s","SIDXAcquisitionGetFrames",message
		return 0
	EndIf
		
End

///// Fix this up so is independent of the Frame Viewer ////////////
Function GetFocusState()
	Wave FrameToShow = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow
	Variable pressed = GetKeyState(0)

	SVAR/Z currentRangeMode = root:Globals:imaging:ImageAnalysisGlobals:currentRangeMode
	NVAR/Z nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	NVAR/Z nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax

	String mode
	String minval,maxval	
	String inwave_string="root:Globals:imaging:ImageAnalysisGlobals:FrameToShow"


	If(SVAR_exists(currentRangeMode))
		mode=StrVarOrDefault(currentRangeMode,"Auto")	
	EndIf
	
	String rangeVals
	String nzmin_S=""
	String nzmax_S=""

	If(pressed ==1)
		mode="ninety_four"
		rangeVals = AR(inwave_string,mode)
		nzmin_S=StringFromList(0,rangeVals,";") 
		nzmax_S=StringFromList(1,rangeVals,";")
		nzmax=Str2num(nzmax_s)
		nzmin=Str2num(nzmin_s)	
	EndIf

	If(pressed==4)
		mode="Auto"
		rangeVals = AR(inwave_string,mode)
		nzmin_S=StringFromList(0,rangeVals,";") 
		nzmax_S=StringFromList(1,rangeVals,";")
		nzmax=Str2num(nzmax_s)
		nzmin=Str2num(nzmin_s)
	EndIf

	If(pressed==5)

		mode="SD2"
		rangeVals = AR(inwave_string,mode)
		nzmin_S=StringFromList(0,rangeVals,";") 
		nzmax_S=StringFromList(1,rangeVals,";")
		nzmax=Str2num(nzmax_s)
		nzmin=Str2num(nzmin_s)
	EndIf

	AdjustImageRange("FrameViewer_subwin",nzmin_S,nzmax_S,inwave_string)



End

////////////////////// NEED TO CLEAN THIS UP /////////////////////////////
/////////////////////// LESS DEP ON GLOBALS ////////////////////////////

Function LightOn()

	NVAR TillLampPresent =root:Globals:physiology:Run_globals:TillLampPresent
	NVAR IlluminationChannel = root:Globals:physiology:Run_globals:IlluminationChannel
	NVAR TillLamp_Lambda = root:Globals:physiology:Run_globals:TillLamp_Lambda


	If(TillLampPresent)
		DeliverTillWave(IlluminationChannel,TillLamp_Lambda)
	Else
		Variable TTL_chan = 2^IlluminationChannel
		String AssertVoltageHigh= "ITC18WriteDigital1 " + num2str(TTL_chan)	
		Execute AssertVoltageHigh
	
	EndIf


End

Function LightOff()

	NVAR TillLampPresent =root:Globals:physiology:Run_globals:TillLampPresent
	NVAR IlluminationChannel = root:Globals:physiology:Run_globals:IlluminationChannel
	NVAR TillLamp_Lambda = root:Globals:physiology:Run_globals:TillLamp_Lambda


	If(TillLampPresent)
		DeliverTillWave(IlluminationChannel,720)
	Else
		String AssertVoltageLow = "ITC18WriteDigital1 0" 
		Execute AssertVoltageLow	
	EndIf
End


////////////////////////// WINDOWS AND BUTTONS: /////////////////////////////////////////

Window focusControl() : Panel
MakeCameraGlobals()
MakeCameraParameterGlobals()
	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(804,66,1234,407)
	SetDrawLayer UserBack
	DrawRect 291,58,45,4
	SetDrawEnv fsize= 15,textrgb= (65280,0,0)
	DrawText 53,95,"Parameters"
	Button button0,pos={62,13},size={100,35},proc=ButtonProc_2,title="Intialize camera"
	Button button1,pos={175,13},size={100,35},proc=ButtonProc_1,title="close camera"
	SetVariable setvar0,pos={54,101},size={87,16},title="Bin x"
	SetVariable setvar0,value= root:Globals:imaging:CameraParameterGlobals:X_pixels
	SetVariable setvar1,pos={54,125},size={93,16},title="Bin y"
	SetVariable setvar1,value= root:Globals:imaging:CameraParameterGlobals:Y_pixels
	SetVariable setvar2,pos={54,149},size={112,16},title="exposure"
	SetVariable setvar2,value= root:Globals:imaging:CameraParameterGlobals:exposure_time
	Button button2,pos={194,120},size={97,50},proc=ButtonProc,title="focus",fSize=15
	SetVariable setvar3,pos={54,173},size={139,16},title="Number of frames"
	SetVariable setvar3,value= root:Globals:imaging:CameraParameterGlobals:num_frames
	Button button3,pos={191,64},size={100,51},proc=ButtonProc_3,title="Collect Sweep"
	Button button3,fSize=15
	SetVariable setvar4,pos={55,198},size={133,16},title="name of image"
	SetVariable setvar4,value= nameOfImage
EndMacro

////////////// The "CLOSE CAMERA" button ////////////////////////
Function CloseCamera_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			deallocateCamera()
			break
	endswitch

	return 0
End

/////////////////// The "INITIALIZE CAMERA " button ////////////////////
Function InitializeCamera_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			initializeCamera()
			break
	endswitch

	return 0
End



////////////////////////////// The Collect Sweep Button ////////////////////////////////
Function CollectOneSweep_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	NVAR totalFrames = root:Globals:imaging:CameraParameterGlobals:num_frames
//	NVAR totalROIs = root:Globals:imaging:CameraParameterGlobals:totalROIs
	SVAR wave_image = root:Globals:physiology:Run_globals:GroupName		///////////// THIS NEEDS TO GO ///////////
	
	Variable totalROIs 
	
	switch( ba.eventCode )
		case 2: // mouse up
			LightOn()	//XXY
			AcquireOneSweepz(totalFrames,wave_image,totalROIs)
			LightOff()
			break
	endswitch

	return 0
End

////////////// The FOCUS BUTTON //////////////////////////
Function Focus_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			fastFocus()
			break
	endswitch

	return 0
End



