#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.0

//

// Prepare DAC and ADC strings for acquisition via ITC18 
////////////////////////////////////// 
Function InitializeStringsForSweep()

	//.....GLOBALS.....//
	SVAR DAC_string_preBuffer = root:Globals:physiology:DAC_ADC_globals:DAC_string_preBuffer
	SVAR ADC_string_preBuffer = root:Globals:physiology:DAC_ADC_globals:ADC_string_preBuffer
	SVAR ADCString_out = root:Globals:physiology:DAC_ADC_globals:ADCString_out
	SVAR DACString_out = root:Globals:physiology:DAC_ADC_globals:DACString_out
	SVAR DACgainString = root:Globals:physiology:DAC_ADC_globals:DACgainString
	SVAR ADCgainString = root:Globals:physiology:DAC_ADC_globals:ADCgainString
	SVAR DACGainString_out = root:Globals:physiology:DAC_ADC_globals:DACGainString_out
	SVAR ADCGainString_out = root:Globals:physiology:DAC_ADC_globals:ADCGainString_out
	//........................//

	// Nuke the zeros from the gain strings (0 indicates that channel is off and not represented in the DAC/ADC string)
	String DACgainString_trimmed = ZapFromString(DACgainString,"0")
	String ADCgainString_trimmed = ZapFromString(ADCgainString,"0")


	UpdateDAC_combo_list()
	UpdateTTL_combo_list()
	BitwiseAddTTLs()

	// EqualizeStrings (below) should take care of most everything that would occur during an
	// experiment, but just to be thorough, the 3 pathological cases are
	// dealt with below. 

	EqualizeITCStrings2(DAC_string_preBuffer,ADC_string_preBuffer)

	// CASE 1: The ADC string is zero, and the DAC string has at least one char
	If(strlen(DAC_String_preBuffer)>0 && strlen(ADC_String_preBuffer)==0)
		ADCString_out=StringOfZeros(strlen(DAC_String_preBuffer))			//Pad the ADC out string with zeros
		ADCGainString_trimmed=StringOfZeros(strlen(DAC_String_preBuffer))	// .. and set all ADC gains to zero

		// CASE 2: The DAC string is zero, and the ADC string has at least one char 
	ElseIf(strlen(ADC_String_preBuffer)>0 && strlen(DAC_String_preBuffer)==0)
		DACString_out=StringOfZeros(strlen(ADC_String_preBuffer))			// Same solution as above...
		DACGainString_trimmed=StringOfZeros(strlen(ADC_String_preBuffer))

		// CASE 3: Both DAC and ADC sequence strings are zero	
	ElseIf(strlen(DAC_String_preBuffer)==0 && strlen(ADC_String_preBuffer)==0)
		DACString_out="0"
		ADCString_out="0"
		DACGainString_trimmed="0"
		ADCGainString_trimmed="0"
	EndIf
 
	DACGainString_out = ReplicateString(DACgainString_trimmed,strlen(DACString_out)) 
	ADCGainString_out = ReplicateString(ADCgainString_trimmed,strlen(ADCString_out))

End




// In practice, this function takes the "preBuffer" DAC and ADC strings and 
// equalizes them. There are some pathological cases to consider (i.e. string
// lengths of zero), but these are mostly handled by the fxn above.
//////////////////////////////////////
Function EqualizeITCStrings2(DACstring,ADCstring)
	String DACstring
	String ADCstring

	//.....GLOBALS.....//
	SVAR DACString_out = root:Globals:physiology:DAC_ADC_globals:DACString_out
	SVAR ADCString_out = root:Globals:physiology:DAC_ADC_globals:ADCString_out
	//........................//

	Variable DAC_length=strlen(DACstring)
	Variable ADC_length=strlen(ADCstring)
	Variable DAC_difference=ADC_length-DAC_length
	Variable extra_DAC
		
	If(DAC_difference==0) 	// Are the sequence strings the same length? ...
		DACstring_out=DACstring	// .. if so, don't touch them
		ADCstring_out=ADCstring
		If(StringMatch(DACString,"") & StringMatch(ADCString,"")) 	// If they're the same length, but that length is zero,  
			DACString_out="0"		//.. then send out "0" as dummy channels for D->A and A->D
			ADCString_out="0"
		EndIf
		extra_DAC=0
		
	Elseif(DAC_difference>0) 	// Is the ADC string longer than the DAC string?... 
		If(strlen(DACstring)==0)	// If the ADC string is longer, but the DAC string is empty.... 
			DACstring_out = PadString(DACstring,ADC_length,char2num("0")) // .. then fill the DAC string with zeros.
		Else
			DACstring_out = ReplicateString(DACstring,ADC_length) // If the DAC string is not empty, then "replicate" it. 
		EndIf
		
		ADCstring_out=ADCstring	
		extra_DAC=DAC_difference
		
	Elseif(DAC_difference<0) 	// Is the DAC string longer than the ADC string? 
		DACstring_out=DACstring		
		If(strlen(ADCstring)==0)	// logic is same as the "Elseif" above
			ADCString_out = PadString(ADCString,DAC_length,char2num("0"))
		Else
			ADCString_out = ReplicateString(ADCstring,DAC_length)
		EndIf
		
		extra_DAC=0
		
	Endif
	
	Return extra_DAC	// This is used as an argument to the multiplex function
End



// Shuffles around some waves that specify which ttls are on, and which channel
// values to use for the bitwise addition. Also adds the ttl combo wave to the 
// list of waves to be multiplexed
//////////////////////////////////////
Function UpdateTTL_combo_list()

	//.....GLOBALS.....//
	Wave/T TTL_combo_list = root:Globals:physiology:DAC_ADC_globals:TTL_combo_list	/// !!!!!!!!!!!!!!!!!!!!!!!wwill need to change these to have full path to wave...//// 
	Wave/T TTL_all_wavelist = root:Globals:physiology:DAC_ADC_globals:TTL_all_wavelist
	Wave/T DAC_combo_list = root:Globals:physiology:DAC_ADC_globals:DAC_combo_list

	Wave TTL_Checklist_wave = root:Globals:physiology:DAC_ADC_globals:TTL_Checklist_wave
	Wave TTL_channels = root:Globals:physiology:DAC_ADC_globals:TTL_channels

	NVAR TTL_isOn=root:Globals:physiology:DAC_ADC_globals:TTL_isOn
	//.........................//

	String last_in_list=""

	Variable n=0
	Variable j=0

	Redimension/N = (sum(TTL_Checklist_wave)) TTL_combo_list,TTL_channels // Redimension to equal the total number of TTLs ON

	Do 
		If (TTL_Checklist_wave[n]==1)		// cycle through channels and make the jth value in the combo list = to nth in the wavelist
			TTL_combo_list[j]=TTL_all_wavelist[n]
			TTL_channels[j]=n			// must be equal to the channel number of the TTL that is ON for bitwise addition
			j+=1
		Endif
		n+=1
	While (j<sum(TTL_Checklist_wave))

	//// Below is the DAC updating that needs to be done when TTLs are updated /// 
	
	If(numpnts(DAC_combo_list)>0)
		last_in_list = DAC_combo_list[numpnts(DAC_combo_list)-1]
	Else
		last_in_list = ""
	Endif
	

	If(!TTL_isOn && (stringmatch(last_in_list,"TTL_combo_wave")))		// Append "TTL_combo_wave" to the list for multiplexing
		DeletePoints (numpnts(DAC_combo_list)-1),1,DAC_combo_list // if at least one TTL is on
	Elseif(TTL_isOn && (!stringmatch(last_in_list,"TTL_combo_wave")))
		InsertPoints (numpnts(DAC_combo_list)),1,DAC_combo_list
		DAC_combo_list[numpnts(DAC_combo_list)-1]="TTL_combo_wave"
	Endif
			
End


// Updates the DAC combo list, which is list of waves going out the ITC.
// If at least one TTL is on, TTL_combo_wave (a single multiplexed binary wave) is put in as the 
// last entry of this list. 
//////////////////////////////////////
Function UpdateDAC_combo_list()

	////.....GLOBALS.....//
	Wave/T DAC_combo_list = root:Globals:physiology:DAC_ADC_globals:DAC_combo_list
	Wave/T DAC_All_wavelist = root:Globals:physiology:DAC_ADC_globals:DAC_All_wavelist
	Wave DAC_Checklist_wave = root:Globals:physiology:DAC_ADC_globals:DAC_Checklist_wave

	NVAR TTL_isOn=root:Globals:physiology:DAC_ADC_globals:TTL_isOn
	SVAR DAC_String_preBuffer = root:Globals:physiology:DAC_ADC_globals:DAC_String_preBuffer
	////.........................//

	String last_in_list=""
	Variable tally=0
	Variable k=0

	Do
		If(DAC_checklist_wave[k]!=0)	// Cycle through the DACs. If the kth DAC is on, increase the total number of active DACs
			tally+=1
		Endif
		k+=1
	While(k<dimsize(DAC_checklist_wave,0))
	
	Redimension/N = (tally) DAC_combo_list
 
	Variable n=0
	Variable j=0
  
	Do 
		If (DAC_Checklist_wave[n]!=0)	//  For the combo list of size = "tally" above, but the output wave in the appropriate slot
			DAC_combo_list[j]=DAC_all_wavelist[n]
			j+=1
		Endif
		n+=1
		
	While(j<tally)		

	If(numpnts(DAC_combo_list)>0)
		last_in_list = DAC_combo_list[numpnts(DAC_combo_list)-1]
	Else
		last_in_list = ""
	Endif	
	
	// Add the TTL combo wave to the multiplexing list as appropriate 
	If(!TTL_isOn && (stringmatch(last_in_list,"TTL_combo_wave")))	
		DeletePoints (numpnts(DAC_combo_list)-1),1,DAC_combo_list
	Elseif(TTL_isOn && (!stringmatch(last_in_list,"TTL_combo_wave")))
		InsertPoints (numpnts(DAC_combo_list)),1,DAC_combo_list
		DAC_combo_list[numpnts(DAC_combo_list)-1]="TTL_combo_wave"	
	Endif
End



// Updates the DAC combo list, which is list of waves going out the ITC
// If at least one TTL is on, TTL_combo_wave (a binary wave) is put in as the 
// last entry of this list. 
//////////////////////////////////////
Function BitwiseAddTTLs()

	//.....GLOBALS.....//
	Wave/T TTL_combo_list = root:Globals:physiology:DAC_ADC_globals:TTL_combo_list
	Wave TTL_Checklist_wave = root:Globals:physiology:DAC_ADC_globals:TTL_Checklist_wave
	Wave TTL_channels = root:Globals:physiology:DAC_ADC_globals:TTL_channels
	//.........................//

	// Find the longest (x dimension) wave in the TTL_combo list
	Make/O/N=(dimsize(TTL_combo_list,0)) TTL_length_sort
	TTL_length_sort[0,*]=dimsize($TTL_combo_list,0)
	Wavestats/Z/Q TTL_length_sort
	Variable max_lenX=V_max
		
	// Find the longest (y dimension) wave in the TTL_combo list
	TTL_length_sort[0,*]=dimsize($TTL_combo_list,1)
	Wavestats/Z/Q TTL_length_sort
	Variable max_lenY=V_max

	// We want out wave to accomodate the longest x and y dimensions in the list... 
	Make/O/N=(max_lenX,max_lenY) TTL_fullwave
	TTL_Fullwave=0

	Variable n
	Variable j

	// Build the combo wave iteratively. For each channel on, add the corresponding channel's wave.
	// The wave has to indicate which channel it "came from", hence the 2^ (channel number)

	For(n=0;n<dimsize(TTL_combo_list,0);n+=1)
		Duplicate/O $TTL_combo_list[n] ttl_wave1
		Make/O ttl_wv
		Redimension/N=(max_lenX,max_lenY) ttl_wv
		RedimWave(ttl_wave1, max_lenX,max_lenY)
		Wave Redim_wave
		TTL_fullwave[0,*][0,*]=TTL_fullwave+Redim_wave[p][q]*(2^(TTL_channels[n]))
		Duplicate/O TTL_fullwave TTL_combo_wave

	EndFor

	Killwaves TTL_fullwave,ttl_wave1,ttl_wv,Redim_Wave,TTL_length_sort
End


// Pulldown menu control for TTLs: adds the appropriate wave only if the appropriate 
// box is checked
//////////////////////////////////////

//////////////////////////////// A temporary panel that I've been using for testing string switching utilities ///////////////////////
///////////// Keeping it around cuz its useful for troubleshooting ////////////////////////////
//Window DAC_ADC_controls() : Panel
//
//	MakeDAC_ADC_Globals()
//
//	PauseUpdate; Silent 1		// building window...
//	NewPanel /W=(511,123,1210,743) as "D/A controls"
//	SetDrawLayer UserBack
//	SetDrawEnv fsize= 16,fstyle= 1,textrgb= (65280,0,0)
//	DrawText 394,20,"ADC inputs"
//	SetDrawEnv fsize= 16,fstyle= 1,textrgb= (0,34816,52224)
//	DrawText 29,20,"DAC Outputs"
//	SetDrawEnv fsize= 16,fstyle= 1,textrgb= (0,34816,52224)
//	DrawText 23,296,"D/A Wavelist"
//	SetDrawEnv fsize= 16,fstyle= 1,textrgb= (0,39168,0)
//	DrawText 23,420,"TTL Combo List"
//	CheckBox ADC7,pos={440,149},size={46,14},proc=ADC_Checklist_proc,title="ADC7"
//	CheckBox ADC7,value= 0
//	CheckBox ADC6,pos={440,115},size={46,14},proc=ADC_Checklist_proc,title="ADC6"
//	CheckBox ADC6,value= 0
//	CheckBox ADC5,pos={440,81},size={46,14},proc=ADC_Checklist_proc,title="ADC5"
//	CheckBox ADC5,value= 0
//	CheckBox ADC4,pos={440,47},size={46,14},proc=ADC_Checklist_proc,title="ADC4"
//	CheckBox ADC4,value= 0
//	CheckBox ADC3,pos={367,149},size={46,14},proc=ADC_Checklist_proc,title="ADC3"
//	CheckBox ADC3,value= 0
//	CheckBox ADC1,pos={367,81},size={46,14},proc=ADC_Checklist_proc,title="ADC1"
//	CheckBox ADC1,value= 0
//	CheckBox ADC2,pos={367,115},size={46,14},proc=ADC_Checklist_proc,title="ADC2"
//	CheckBox ADC2,value= 0
//	CheckBox ADC0,pos={367,47},size={46,14},proc=ADC_Checklist_proc,title="ADC0"
//	CheckBox ADC0,value= 0
//	PopupMenu DAC0,pos={32,45},size={87,21},proc=DAC_Menu_popProc,title="DAC 0"
//	PopupMenu DAC0,mode=1,popvalue="Off",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
//	PopupMenu dac0wave0,pos={67,68},size={169,21},proc=PopMenuProc,title="DAC wave 0"
//	PopupMenu dac0wave0,mode=1,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
//	PopupMenu dac1wav1,pos={68,118},size={169,21},proc=PopMenuProc,title="DAC wave 1"
//	PopupMenu dac1wav1,mode=1,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
//	PopupMenu dac2wave2,pos={69,169},size={169,21},proc=PopMenuProc,title="DAC wave 2"
//	PopupMenu dac2wave2,mode=1,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
//	PopupMenu dac3wave3,pos={68,220},size={169,21},proc=PopMenuProc,title="DAC wave 3"
//	PopupMenu dac3wave3,mode=1,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
//	PopupMenu DAC1,pos={31,94},size={87,21},proc=DAC_Menu_popProc,title="DAC 1"
//	PopupMenu DAC1,mode=1,popvalue="Off",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
//	PopupMenu DAC2,pos={31,143},size={87,21},proc=DAC_Menu_popProc,title="DAC 2"
//	PopupMenu DAC2,mode=1,popvalue="Off",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
//	PopupMenu DAC3,pos={29,191},size={87,21},proc=DAC_Menu_popProc,title="DAC 3"
//	PopupMenu DAC3,mode=1,popvalue="Off",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
//	CheckBox TTL0,pos={283,309},size={47,14},proc=TTL_Checklist_proc,title="TTL 0"
//	CheckBox TTL0,value= 0
//	CheckBox TTL1,pos={283,337},size={47,14},proc=TTL_Checklist_proc,title="TTL 1"
//	CheckBox TTL1,value= 0
//	CheckBox TTL2,pos={283,365},size={47,14},proc=TTL_Checklist_proc,title="TTL 2"
//	CheckBox TTL2,value= 0
//	CheckBox TTL3,pos={283,393},size={47,14},proc=TTL_Checklist_proc,title="TTL 3"
//	CheckBox TTL3,value= 0
//	PopupMenu TTLW0,pos={355,306},size={144,21},proc=PopMenuProc_1,title="TTL wave 0"
//	PopupMenu TTLW0,mode=1,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
//	PopupMenu TTLW1,pos={355,334},size={144,21},proc=PopMenuProc_1,title="TTL wave 1"
//	PopupMenu TTLW1,mode=1,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
//	PopupMenu TTLW2,pos={355,362},size={144,21},proc=PopMenuProc_1,title="TTL wave 2"
//	PopupMenu TTLW2,mode=1,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
//	PopupMenu TTLW3,pos={355,390},size={144,21},proc=PopMenuProc_1,title="TTL wave 3"
//	PopupMenu TTLW3,mode=1,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
//	SetVariable setvar0,pos={323,220},size={288,23},labelBack=(65280,16384,16384)
//	SetVariable setvar0,fSize=16,fStyle=1
//	SetVariable setvar0,value= root:Globals:physiology:DAC_ADC_globals:ADC_string_preBuffer
//	SetVariable setvar1,pos={323,193},size={289,23},labelBack=(0,34816,52224)
//	SetVariable setvar1,fSize=16,fStyle=1
//	SetVariable setvar1,value= root:Globals:physiology:DAC_ADC_globals:DAC_string_preBuffer
//	ListBox list0,pos={23,308},size={167,86}
//	ListBox list0,listWave=root:Globals:physiology:DAC_ADC_globals:DAC_combo_list
//	ListBox list1,pos={23,427},size={168,87}
//	ListBox list1,listWave=root:Globals:physiology:DAC_ADC_globals:TTL_combo_list
//	Button Multiplex,pos={287,438},size={184,86},proc=ButtonProc,title="Multiplex"
//	Button Multiplex,fSize=20
//	SetVariable delta_t,pos={287,535},size={134,16},title="delta_t"
//	SetVariable delta_t,value= root:Globals:physiology:DAC_ADC_globals:delta_t
//	SetVariable sweep_number,pos={286,556},size={143,16},title="sweep number"
//	SetVariable sweep_number,value= root:Globals:physiology:DAC_ADC_globals:swp_number
//	SetVariable points_to_acquire,pos={287,580},size={148,16},title="points to acquire"
//	SetVariable points_to_acquire,value= root:Globals:physiology:DAC_ADC_globals:points_to_acquire
//EndMacro
//

////////////////// Most Recent: 9/24/07.... /////////////////////

// Makes the panel for controlling the DAC, ADC, TTL in/outs.
//////////////////////////////////////
Function AcqMakeSwitchboard() 
	InitializeExperiment()
	
	DoWindow/F Switchboard
	If(V_Flag==1)
		return 0
	EndIf

	PauseUpdate; Silent 1		// building window...
	NewPanel/N=Switchboard /W=(302,114,671,352)/N=Switchboard as "Switchboard"
	MakeDAC_ADC_globals()
	TabControl tab0,pos={6,9},size={353,192},proc=TabProc,tabLabel(0)="D/A OUT"
	TabControl tab0,tabLabel(1)="A/D IN",tabLabel(2)="TTLs",tabLabel(3)="Protocols"
	TabControl tab0,value= 0
	TitleBox Gain_tab0,pos={83,38},size={36,24},title="Gain",fSize=14,frame=2
	TitleBox Gain_tab0,fColor=(0,15872,65280)
	TitleBox Stim_tab0,pos={212,38},size={93,24},title="Stimulus wave",fSize=14
	TitleBox Stim_tab0,frame=2,fColor=(0,15872,65280)
	TitleBox Gain_tab1,pos={106,39},size={36,24},disable=1,title="Gain",fSize=14
	TitleBox Gain_tab1,frame=2,fColor=(0,15872,65280)
	TitleBox Range_tab1,pos={26,39},size={49,24},disable=1,title="Range",fSize=14
	TitleBox Range_tab1,frame=2,fColor=(0,15872,65280)
	GroupBox GainBox_tab0,pos={29,65},size={149,114},labelBack=(52224,52224,52224)
	GroupBox GainBox_tab0,fColor=(65535,65535,65535)
	PopupMenu DAC0_tab0,pos={35,70},size={137,21},proc=DAC_PopupControl,title="Ch 0"
	PopupMenu DAC0_tab0,fSize=10
	PopupMenu DAC0_tab0,mode=2,bodyWidth= 110,popvalue="MC Current",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
	PopupMenu DAC1_tab0,pos={35,97},size={137,21},proc=DAC_PopupControl,title="Ch 1"
	PopupMenu DAC1_tab0,fSize=10
	PopupMenu DAC1_tab0,mode=3,bodyWidth= 110,popvalue="MC Voltage",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
	PopupMenu DAC2_tab0,pos={35,124},size={137,21},proc=DAC_PopupControl,title="Ch 2"
	PopupMenu DAC2_tab0,fSize=10
	PopupMenu DAC2_tab0,mode=1,bodyWidth= 110,popvalue="Off",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
	PopupMenu DAC3_tab0,pos={35,152},size={137,21},proc=DAC_PopupControl,title="Ch 3"
	PopupMenu DAC3_tab0,fSize=10
	PopupMenu DAC3_tab0,mode=1,bodyWidth= 110,popvalue="Off",value= #"\"Off;MC Current;MC Voltage;Till Wavelength\""
	GroupBox WaveBox_tab0,pos={183,65},size={155,114},labelBack=(52224,52224,52224)
	GroupBox WaveBox_tab0,fColor=(65535,65535,65535)
	PopupMenu dac0wave0_tab0,pos={190,70},size={140,21},proc=DACWaveSelect_Pop
	PopupMenu dac0wave0_tab0,mode=5,bodyWidth= 140,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
	PopupMenu dac1wav1_tab0,pos={191,97},size={140,21},proc=DACWaveSelect_Pop
	PopupMenu dac1wav1_tab0,mode=12,bodyWidth= 140,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
	PopupMenu dac2wave2_tab0,pos={190,124},size={140,21},proc=DACWaveSelect_Pop
	PopupMenu dac2wave2_tab0,mode=6,bodyWidth= 140,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
	PopupMenu dac3wave3_tab0,pos={190,152},size={140,21},proc=DACWaveSelect_Pop
	PopupMenu dac3wave3_tab0,mode=8,bodyWidth= 140,popvalue="zero_outwave",value= #"SORTLIST(Wavelist(\"*_outwave\", \";\", \"\" ),\";\",4)"
	GroupBox ADCbox1_tab1,pos={19,66},size={161,125},disable=1
	GroupBox ADCbox1_tab1,labelBack=(52224,52224,52224)
	PopupMenu ADC0_tab1,pos={82,77},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC0_tab1,fSize=10
	PopupMenu ADC0_tab1,mode=3,bodyWidth= 91,popvalue="MC voltage",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC1_tab1,pos={82,104},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC1_tab1,fSize=10
	PopupMenu ADC1_tab1,mode=2,bodyWidth= 91,popvalue="MC current",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC2_tab1,pos={82,132},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC2_tab1,fSize=10
	PopupMenu ADC2_tab1,mode=1,bodyWidth= 91,popvalue="Off",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC3_tab1,pos={82,160},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC3_tab1,fSize=10
	PopupMenu ADC3_tab1,mode=1,bodyWidth= 91,popvalue="Off",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC0_gain_tab1,pos={23,77},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 0"
	PopupMenu ADC0_gain_tab1,fSize=10,fStyle=0
	PopupMenu ADC0_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	PopupMenu ADC1_gain_tab1,pos={23,104},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 1"
	PopupMenu ADC1_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	PopupMenu ADC2_gain_tab1,pos={23,132},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 2"
	PopupMenu ADC2_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	PopupMenu ADC3_gain_tab1,pos={23,160},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 3"
	PopupMenu ADC3_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	GroupBox ADCbox2_tab1,pos={188,66},size={161,125},disable=1
	GroupBox ADCbox2_tab1,labelBack=(52224,52224,52224)
	PopupMenu ADC4_tab1,pos={253,77},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC4_tab1,fSize=10
	PopupMenu ADC4_tab1,mode=1,bodyWidth= 91,popvalue="Off",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC5_tab1,pos={253,104},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC5_tab1,fSize=10
	PopupMenu ADC5_tab1,mode=1,bodyWidth= 91,popvalue="Off",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC6_tab1,pos={253,132},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC6_tab1,fSize=10
	PopupMenu ADC6_tab1,mode=1,bodyWidth= 91,popvalue="Off",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC7_tab1,pos={253,160},size={91,21},disable=1,proc=ADC_PopupControl
	PopupMenu ADC7_tab1,fSize=10
	PopupMenu ADC7_tab1,mode=1,bodyWidth= 91,popvalue="Off",value= #"\"Off;MC current;MC voltage\""
	PopupMenu ADC4_gain_tab1,pos={194,77},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 4"
	PopupMenu ADC4_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	PopupMenu ADC5_gain_tab1,pos={194,104},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 5"
	PopupMenu ADC5_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	PopupMenu ADC6_gain_tab1,pos={194,132},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 6"
	PopupMenu ADC6_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	PopupMenu ADC7_gain_tab1,pos={194,160},size={59,21},disable=1,proc=SetADCrangePop,title="Ch 7"
	PopupMenu ADC7_gain_tab1,mode=0,value= #"root:Globals:physiology:DAC_ADC_globals:ADCRanges"
	TitleBox Range2_tab1,pos={197,39},size={49,24},disable=1,title="Range",fSize=14
	TitleBox Range2_tab1,frame=2,fColor=(0,15872,65280)
	TitleBox Gain2_tab1,pos={276,39},size={36,24},disable=1,title="Gain",fSize=14
	TitleBox Gain2_tab1,frame=2,fColor=(0,15872,65280)
	TitleBox Stimz_tab2,pos={148,35},size={93,24},disable=1,title="Stimulus wave"
	TitleBox Stimz_tab2,fSize=14,frame=2,fColor=(0,15872,65280)
	TitleBox Stim_tab2,pos={71,35},size={60,24},disable=1,title="TTL On?",fSize=14
	TitleBox Stim_tab2,frame=2,fColor=(0,15872,65280)
	GroupBox TTLBox_tab2,pos={70,65},size={204,129},disable=1
	GroupBox TTLBox_tab2,labelBack=(52224,52224,52224),fColor=(52224,52224,52224)
	CheckBox TTL1_tab2,pos={83,109},size={24,14},disable=1,proc=TTL_Checklist_proc,title="1"
	CheckBox TTL1_tab2,value= 0
	CheckBox TTL2_tab2,pos={83,136},size={24,14},disable=1,proc=TTL_Checklist_proc,title="2"
	CheckBox TTL2_tab2,value= 0
	CheckBox TTL3_tab2,pos={83,164},size={24,14},disable=1,proc=TTL_Checklist_proc,title="3"
	CheckBox TTL3_tab2,value= 0
	PopupMenu TTLW0_tab2,pos={120,80},size={140,21},disable=1,proc=TTL_popup_Proc
	PopupMenu TTLW0_tab2,mode=1,bodyWidth= 140,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
	PopupMenu TTLW1_tab2,pos={120,107},size={140,21},disable=1,proc=TTL_popup_Proc
	PopupMenu TTLW1_tab2,mode=3,bodyWidth= 140,popvalue="trigs_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
	PopupMenu TTLW2_tab2,pos={120,133},size={140,21},disable=1,proc=TTL_popup_Proc
	PopupMenu TTLW2_tab2,mode=1,bodyWidth= 140,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
	PopupMenu TTLW3_tab2,pos={120,160},size={140,21},disable=1,proc=TTL_popup_Proc
	PopupMenu TTLW3_tab2,mode=1,bodyWidth= 140,popvalue="siu_pulse",value= #"SORTLIST(Wavelist(\"*_pulse\", \";\", \"\" ),\";\",4)"
	CheckBox TTL0_tab2,pos={84,82},size={24,14},disable=1,proc=TTL_Checklist_proc,title="0"
	CheckBox TTL0_tab2,value= 0
	Button DA_Gains_Off_tab0,pos={45,206},size={117,26},proc=All_DA_GainsOff_button,title="All channels off"
	Button All_AD_Off_tab1,pos={45,206},size={117,26},disable=1,proc=All_AD_GainsOff_button,title="All channels off"
	Button All_ADRanges_10_tab1,pos={175,206},size={117,26},disable=1,proc=SetADCranges_10_Pop,title="All Ranges 10 V"
	Button AllTTLs_Off_tab2,pos={45,206},size={117,26},disable=1,proc=TTL_Off_button,title="All TTLs Off"
	GroupBox Electrode1_tab3,pos={19,73},size={79,65},disable=1
	GroupBox Electrode1_tab3,labelBack=(52224,52224,52224),font="Arial",fSize=13
	Button SealFirst_tab3,pos={28,78},size={60,25},disable=1,proc=ProtocolProc,title="Seal "
	Button IClampFirst_tab3,pos={28,108},size={60,25},disable=1,proc=ProtocolProc,title="I clamp"
	GroupBox Electrode2_tab3,pos={121,73},size={79,65},disable=1
	GroupBox Electrode2_tab3,labelBack=(52224,52224,52224),font="Arial",fSize=13
	Button SealSecond_tab3,pos={131,78},size={60,25},disable=1,proc=ProtocolProc,title="Seal"
	Button IClampSecond_tab3,pos={131,108},size={60,25},disable=1,proc=ProtocolProc,title="I Clamp "
	TitleBox El1_tab3,pos={20,39},size={76,24},disable=1,title="Electrode 1"
	TitleBox El1_tab3,fSize=14,frame=2,fColor=(0,0,62720)
	TitleBox El2_tab3,pos={121,39},size={76,24},disable=1,title="Electrode 2"
	TitleBox El2_tab3,fSize=14,frame=2,fColor=(0,0,62720)
	TitleBox Pairs_tab3,pos={261,39},size={39,24},disable=1,title="Pairs",fSize=14
	TitleBox Pairs_tab3,frame=2,fColor=(0,0,62720)
	GroupBox Pair_tab3,pos={223,73},size={120,65},disable=1
	GroupBox Pair_tab3,labelBack=(52224,52224,52224),font="Arial",fSize=13
	Button Iclamp1_Vclamp2_tab3,pos={238,78},size={88,25},disable=1,proc=ProtocolProc,title="Iclamp, Vclamp"
	Button Vclamp1_Iclamp2_tab3,pos={237,108},size={88,25},disable=1,proc=ProtocolProc,title="Vclamp,  Iclamp"
	InitSwitchboard()
End


// Useful Code Snippet (from WM manual) for disabling/enabling 
// tab controls. 
//////////////////////////////////////
Function TabProc_1(ctrlName,tabNum) : TabControl
	String ctrlName
	Variable tabNum
	
	String controlsInATab=ControlNameList("",";","tab*")
	
	
	String curTabMatch="tab"+num2istr(tabNum)+"*"
	String controlsinCurTab=ListMatch(controlsInATab,curTabMatch)
	
	String controlsInOtherTabs=ListMatch(controlsInATab,"!"+curTabMatch)
	ModifyControlList controlsInOtherTabs disable=1
	ModifyControlList controlsInCurTab disable=0
	
	return 0
End



// Make the necessary globals sending ITC commands & mainpulating 
// Strings & wave for DAC
//////////////////////////////////////
Function MakeDAC_ADC_globals()
	String savDF = GetDataFolder(1)
	
	Make/O root:muxed_wave	// write this to root so its easy to check on... 
	
	If(DataFolderExists("root:Globals:physiology:DAC_ADC_globals"))
		SetDataFolder root:Globals:physiology:DAC_ADC_globals
	Else
		NewDataFolder/S root:Globals:physiology:DAC_ADC_globals
		If(exists("DAC_string_preBuffer")==0)

			// MAKE GLOBAL STRINGS
			String/G DAC_string_preBuffer =""
			String/G ADC_string_preBuffer =""

			String/G DACstring_out =""
			String/G ADCstring_out =""

			String/G DACgainString ="0000"
			String/G ADCgainString ="00000000"
			String/G DACGainString_out = ""
			String/G ADCGainString_out = ""
			String/G ADCRanges=  "10V;5V;2V;1V"

			// MAKE GLOBAL VARIABLES	
			Variable/G TTL_isOn = 0

			// MAKE "GLOBAL" WAVES
			Make/T/N=1 DAC_combo_list 
			Make/T/N=4 DAC_all_wavelist
			Make/N=4 DAC_checklist_wave

			Make/T/N=0 TTL_combo_list
			Make/T/N=4 TTL_all_wavelist
			Make/N=4 TTL_checklist_wave
			// Make TTL_combo_wave // don't think this is necessary... we'll see 
			Make/N=0 TTL_channels 
			Make/N=4 output_gain_lookup; output_gain_lookup = {0,1.5,166.8,146.7}
			Make/N=3 input_gain_lookup; input_gain_lookup = {.000625,.000625,.03125}
			Make test_outwave
			Make test_pulse
			// Make Gain_checklist_wave // doesn't seem to be necessary... 
		EndIf
	EndIf		
	SetDataFolder savDF		
End


//////////////////////////////////// UTILITY FUNCTIONS: MANIPULATING STRINGS & WAVES FOR ACQUISITION ////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// A general purpose routine used for redimensioning a wave in a "cyclic" way
// For example, if you have a wave with 3 columns each with different 1-d slice,
// and want to resize it to a 7 column wave, it will keep adding "blocks" of 
// the 3 column wave until it has a total of 7 columns. The columns of the 
// resulting 7d wave will be from columns 0,1,2,0,1,2,0  of the INWAVE
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function RedimWave(inwave, FinaldimX,FinaldimY)
	Wave inwave
	Variable FinaldimX
	Variable FinaldimY

	Variable inwaveColumns = dimsize(inwave,1)
	Variable currentYDim = 0
	Variable n

	Make/O/N=(FinaldimX,FinaldimY) Redim_wave
	SetScale/P x,0,dimdelta(inwave,0), Redim_wave

	If(inwaveColumns==0)
		Duplicate/O inwave tmpwave
		Redimension/N=(FinaldimX,FinaldimY) tmpwave
	
		Do
			Redim_wave[0,*][n]=tmpwave[p]
			n+=1
		While(n<FinalDimY)

	Else
		Do
			Variable ThisColumn=mod(currentYdim,inwaveColumns)
			Duplicate/O/R=[0,*][ThisColumn,ThisColumn] inwave tmpwave
			Redimension/N=(FinaldimX) tmpwave
			Redim_wave[0,*][currentYdim]=tmpwave[p]
			currentYDim+=1		
		While(currentYDim<FinalDimY)
	EndIf


End


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Perhaps not worthy of being a function, but makes some code above more
// readable.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function/S ZapFromString(inString,killString)
	String inString
	String killString
	String outString=""
	outString = replaceString(killString,inString,"")
	//Print outString
	Return OutString
End


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Same deal as above..pretty obvious what it does
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function/S StringOfZeros(length)
	Variable length
	String outString =""

	If(length<=0)
		Return ""
	EndIf
	Do
		outString=outString+"0"
		If(Strlen(outString)==length)
			break
		EndIf
	While(1)	
	Return outString	

End

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Given an inString, produces an outstring of specified length. Characters of the 
// instring are appended cyclically. E.G. ReplicateString("0123",9) would 
// produce "012301230"
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function/S ReplicateString(inString,finalLength)
	String inString
	Variable finalLength
	String outString = inString
	String padChar=""
	Variable j
	String inStringCopy

	If(finalLength==strlen(inString))
		outString = inString
	
	ElseIf(finalLength<strlen(inString))	
		outString = inString[0,finalLength-1]
		
	ElseIf(finalLength>strlen(inString))

		Do
			padChar = inString[mod(j,strlen(inString))]
			outString = outString + padChar
			j+=1
		While (Strlen(outString)<(finalLength))
	EndIf
	Return outString	
	
End

//////////////////////////////////////////////////// BUTTONS, MENUS, POPUPS ///////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/// For the switchboard control to select output waves .
//////////////////////////////////////
Function DACWaveSelect_Pop(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	//.....GLOBALS.....//		
	Wave/T DAC_All_wavelist = root:Globals:physiology:DAC_ADC_globals:DAC_All_wavelist
	Wave/T DAC_combo_list = root:Globals:physiology:DAC_ADC_globals:DAC_combo_list
	Wave DAC_checklist_wave = root:Globals:physiology:DAC_ADC_globals:DAC_checklist_wave
	//.........................// 

	Variable index=strlen(ctrlName)
	//String index_str=ctrlName[index-1]
	String index_str=ctrlName[3]

	DAC_All_wavelist[str2num(index_str)]=popStr
	//	
	//	If(DAC_checklist_wave[str2num(index_str)]!=0)
	//		UpdateDAC_combo_list()
	//	Else
	//		DAC_combo_list = DAC_combo_list
	//	Endif

End

/// For the switchboard control to select TTL outs 
//////////////////////////////////////
Function TTL_popup_Proc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	//.....GLOBALS.....//			
	Wave/T TTL_all_wavelist = root:Globals:physiology:DAC_ADC_globals:TTL_all_wavelist
	Wave TTL_Checklist_wave = root:Globals:physiology:DAC_ADC_globals:TTL_Checklist_wave
	//.........................// 

	//Variable index=str2num(ctrlName[strlen(ctrlName)-1])
	Variable index=str2num(ctrlName[4])
	TTL_all_wavelist[index]=popStr

	///////////////This should go in its own function????????/////////////////////////////	
	//	UpdateTTL_combo_list()
	//	
	//	If (TTL_Checklist_wave(index)==1)
	//		BitwiseAddTTLs() ///////////!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! again.. probably don't want to do this here... might f up multiplexing ////////////
	//	Endif
	//		
End


/// Select Active TTLs 
//////////////////////////////////////
Function TTL_Checklist_proc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	//.....GLOBALS.....//		
	Wave TTL_Checklist_wave=root:Globals:physiology:DAC_ADC_globals:TTL_Checklist_wave

	NVAR TTL_isOn=root:Globals:physiology:DAC_ADC_globals:TTL_isOn
	SVAR DAC_string_preBuffer=root:Globals:physiology:DAC_ADC_globals:DAC_string_preBuffer
	SVAR DACgainString = root:Globals:physiology:DAC_ADC_globals:DACgainString
	//.........................// 	

	//Variable index = str2num(ctrlName[strlen(ctrlName)-1])	
	Variable index = str2num(ctrlName[3])

	If (checked)
		TTL_Checklist_wave[index]=1
		If(strlen(DACgainString)<5) // If the TTL gain entry in DAC gain string isn't on...
			DACgainString = PadString(DACgainString,5,char2num("T")) //.. add it
		EndIf
	Else
		TTL_Checklist_wave[index]=0
		
		If((strlen(DACgainString)>4) & sum(TTL_Checklist_wave)==0) // If the TTL gain entry ("T") isn't already off ...
			DACgainString = RemoveEnding(DACgainString) //.. zap it
			
		EndIf
	Endif
	
	If(mean(TTL_Checklist_wave)>0) // If there is at least one TTL checked and no digital-out indicated in the ADCseq string...
		TTL_isOn = 1
		If(!stringmatch(DAC_string_preBuffer,"*D"))
			DAC_string_preBuffer=DAC_string_preBuffer +"D" //.. add it
		Else
			DAC_string_preBuffer=DAC_string_preBuffer	
		Endif
		
	Elseif(mean(TTL_Checklist_wave)==0) // If there are no TTLs checked and digital out is indicated...
		TTL_isOn=0
		If(stringmatch(DAC_string_preBuffer,"*D"))
			DAC_string_preBuffer=ReplaceString("D",DAC_string_preBuffer,"") //.. zap it
		Else
			DAC_string_preBuffer = DAC_string_preBuffer
		Endif
	Endif
	
	//UpdateTTL_combo_list()
	//BitwiseAddTTLs() // This Should be saved for later..... Maybe right before the multiplexing

End

// Set Voltage range for the ADcs 
//////////////////////////////////////
Function SetADCrangePop(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	String setRange = "ITC18SetADCRange " + ctrlName[3] + "," 
	
	
	strswitch (popStr)
			
			
		case "10V" :
			SetRange=SetRange+"10"
			break
				
		case "5V":
			SetRange=SetRange+"5"
			break
			
		case "2V":
			SetRange=SetRange+"2"
			break
			
		case "1V":
			SetRange=SetRange+"1"
			break
				
			
	endswitch

	Execute(SetRange)
	
End

//////////////////////////////////////
Function SetDACGains_zero(DAC_or_ADC)
	Variable DAC_or_ADC // 0 for DA, 1 for the AD


	String WindowString = WinList("Switchboard",";","")

	If(Stringmatch(WindowString,"*Switchboard*"))

		Variable index
		Variable control_number = (DAC_or_ADC==0) ? 4 : 8 	
	
		For(index=0;index<control_number;index+=1)
			String thisDAC
		
			If(DAC_or_ADC==0)
				thisDAC = "DACX_tab0"
			Else
				thisDAC = "ADCX_tab1"
			EndIf
		 
			thisDAC=ReplaceString("X",thisDAC,num2str(index))
	 	
			ControlInfo/W=Switchboard $thisDAC
			If (!StringMatch(S_value,"Off"))
				PopupMenu $thisDAC mode=1,popvalue="Off",win=Switchboard
				If(DAC_or_ADC==0)
					DAC_PopupControl(thisDAC,1,"Off")
				Else
					ADC_PopupControl(thisDAC,1,"Off")
				EndIf
				
				ControlUpdate/W=Switchboard $thisDAC
			EndIf
			
		
		EndFor
	
	EndIf


End


Function All_DA_GainsOff_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			SetDACGains_zero(0)
			break
	endswitch

	return 0
End

Function All_AD_GainsOff_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			SetDACGains_zero(1)
			break
	endswitch

	return 0
End

Function SetADCranges_10_Pop(ba) :ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 2: // mouse up
						
			All_Ranges_10()
			break
	endswitch
	return 0
End

Function ALL_Ranges_10()
	Variable index = 0
	Do
		String setRange = "ITC18SetADCRange " + num2str(index) + ",10" 
		Execute(setRange)
		index+=1
	While(index<8)

End

Function All_TTLs_Off()

	String WindowString = WinList("Switchboard",";","")

	If(Stringmatch(WindowString,"*Switchboard*"))

		Variable index
		
		For(index=0;index<4;index+=1)
			String thisTTL
		
		
			thisTTL = "TTLX_tab2"
		
		 
			thisTTL=ReplaceString("X",thisTTL,num2str(index))
	 	
			Checkbox $thisTTL, value=0,win=Switchboard
		
	
			TTL_Checklist_proc(thisTTL,0)
		
			ControlUpdate/W=Switchboard $thisTTL
		EndFor

	EndIf

End

Function TTL_Off_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			All_TTLs_Off()
			break
	endswitch

	return 0
End

Function InitSwitchboard()
	All_TTLs_Off()
	All_Ranges_10()
	SetDACGains_zero(0)
	SetDACGains_zero(1)
	Make/O/N=1000 zero_outwave
	

End


Function ADC_PopupControl(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr	

	//.....GLOBALS.....//
	SVAR ADC_string_preBuffer = root:Globals:physiology:DAC_ADC_globals:ADC_string_preBuffer
	SVAR ADCgainString = root:Globals:physiology:DAC_ADC_globals:ADCgainString
	//........................//

	Variable last = strlen(ctrlName)-1
	Variable j
	//String match_string = "*"+ctrlName[last]+"*"
	String match_string = "*"+ctrlName[3]+"*"
	String tempString=""
		
	If (popNum!=1)	// If This ADC is not OFF ..

		If (!stringmatch(ADC_string_preBuffer,match_string))		// If This ADC channel is not represented in the list of outputs
			// in the buffer String
			ADC_string_preBuffer = ADC_string_preBuffer + ctrlName[3] // may need to change
		Else
			ADC_string_preBuffer = ADC_string_preBuffer		// If the channel was already there (It shouldn't be), then do nothing
		Endif
	
	Else			// If the ADC channel is OFF ..
		ADC_string_preBuffer = ReplaceString(ctrlName[3],ADC_string_preBuffer,"") // Remove it from the the Buffer string
	Endif
	
	For(j=0;j<=2*strlen(ADC_string_preBuffer);j+=1)	// Turn the ADC string (i.e. "012") into a list (i.e. "0;1;2;"), which is sorted below
		If(!mod(j,2)==0)
			tempString[j]=";"
		Else
			tempString[j]=ADC_string_preBuffer[j/2]
		Endif
		
	EndFor
	
	tempString=SortList(tempString,";",2)	// Sort the list in ascending numerical order

	ADC_string_preBuffer = ReplaceString(";",tempString,"")	// Remove semicolons from the list ..
	
	//String ctrlNumber = ctrlName[strlen(CtrlName)-1]
	String ctrlNumber = ctrlName[3]
	ADCgainString[str2num(ctrlNumber),str2num(ctrlNumber)]=num2str(popNum-1) // And update the gain of the corresponding channel


	//// Update some variables and waves for the Grapher ///////
	/// THis is an ugly hack. should integrate it more cleanly with the grapher.... 
	
	If(DataFolderExists("root:Globals:physiology:GrapherGlobals"))
		SetADCAvailableList()
		NVAR numGraphs = root:Globals:physiology:GrapherGlobals:numGraphs
		Wave/T ADCAvailableList = root:globals:Physiology:GrapherGlobals:ADCAvailableList
		Wave/T WavesToGraph = root:globals:Physiology:GrapherGlobals:WavesToGraph
		Wave selWave = root:globals:Physiology:GrapherGlobals:selWave
		
		ResizePlotterChoices(numGraphs, ADCAvailableList,WavesToGraph,selWave)

	EndIf

End

//////////////////////////////// These long definitions are protocols ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

Function ProtocolProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	String Protocol = ba.ctrlName
	NVAR SweepLength=root:Globals:physiology:Run_globals:SweepLength
	NVAR SweepInterval=root:Globals:physiology:Run_globals:SweepInterval
	NVAR TotalSweeps = root:Globals:physiology:Run_globals:TotalSweeps
	NVAR ImagingIsOn = 	root:Globals:physiology:Run_globals:ImagingIsOn
	NVAR saveMode = root:Globals:physiology:Run_globals:saveMode
	
	Variable popupMode
	Variable start,step,n 	// For making 2d waves
	String Alloutwaves=""
	
	switch( ba.eventCode )
		case 2: // mouse up
			SetDataFolder root:
				
				
				
			SetDACGains_zero(0)	/// clear all settings first
			SetDACGains_zero(1) /// .. and make sure imaging is off (below)
			ImagingIsOn=0; ControlUpdate/W=main CollectImages_tab0; CheckBox  CollectImages_tab0  win=main,value=0
			All_Ranges_10()
			All_TTLs_Off()
				
			strswitch (protocol)	// ... and set things to appropriate gains, etc. 
				case "SealFirst_tab3":
				
					SweepLength=50
					SweepInterval=0
					TotalSweeps=8000
					saveMode=0
					If(!WaveExists(seal_test_outwave))
						MakePulse(10,1,30,0,2,5,.1,"seal_test_outwave")
					EndIf
					AllOutwaves = Sortlist(Wavelist("*_outwave", ";", "" ),";",4)
					//Print AllOutwaves
					popupMode = WhichListItem("seal_test_outwave", AllOutwaves)		
						
					DAC_PopupControl("DAC0_tab0",3,"MC voltage")
					ControlUpdate/W= Switchboard DAC0_tab0; PopupMenu DAC0_tab0 mode=3
					DACWaveSelect_Pop("dac0wave0_tab0",1,"seal_test_outwave")	
					ControlUpdate/W=Switchboard dac0wave0_tab0; PopupMenu dac0wave0_tab0 mode=popupMode+1
					ADC_PopupControl("ADC0_tab1",2,"MC current")
					ControlUpdate/W= Switchboard ADC0_tab1; PopupMenu ADC0_tab1 mode=2
					ADC_PopupControl("ADC2_tab1",3,"MC voltage")
					ControlUpdate/W= Switchboard ADC2_tab1; PopupMenu ADC2_tab1 mode=3
					
					// Don't save sweeps during seal test...
					
					SaveMode=0
					ControlUpdate/W=Main SaveMode_tab0; PopupMenu SaveMode_tab0 win=Main, mode=1
					
					break
				
				case "SealSecond_tab3":
				
					SweepLength=50
					SweepInterval=0
					TotalSweeps=8000
					saveMode=0
					If(!WaveExists(seal_test_outwave))
						MakePulse(10,1,30,0,2,5,.1,"seal_test_outwave")
					EndIf
					AllOutwaves = Sortlist(Wavelist("*_outwave", ";", "" ),";",4)
					
					popupMode = WhichListItem("seal_test_outwave", AllOutwaves)		
						
					DAC_PopupControl("DAC1_tab0",3,"MC voltage")
					ControlUpdate/W= Switchboard DAC1_tab0; PopupMenu DAC1_tab0 mode=3
					DACWaveSelect_Pop("dac1wav1_tab0",1,"seal_test_outwave")	
					ControlUpdate/W=Switchboard dac1wav1_tab0; PopupMenu dac1wav1_tab0 mode=popupMode+1
					ADC_PopupControl("ADC1_tab1",2,"MC current")
					ControlUpdate/W= Switchboard ADC1_tab1; PopupMenu ADC1_tab1 mode=2
					ADC_PopupControl("ADC3_tab1",2,"MC voltage")
					ControlUpdate/W= Switchboard ADC3_tab1; PopupMenu ADC3_tab1 mode=3
					
					SaveMode=0
					ControlUpdate/W=Main SaveMode_tab0; PopupMenu SaveMode_tab0 win=Main, mode=1
					break
							
				case "IclampFirst_tab3":
				
					SweepLength=1000
					SweepInterval=6
					TotalSweeps=56
					saveMode=1
					start=-50;step=100
					
					If(!WaveExists(IV_2d_outwave))
						Do
							MakePulse(50,1,100,0,5,start+(step*n),.1,"tempwv")
							PackageTo2d($"tempwv","IV_2d_outwave")
							n+=1
						While(n<6)
						n=0
								
					EndIf
					AllOutwaves = Sortlist(Wavelist("*_outwave", ";", "" ),";",4)
					//Print AllOutwaves
					popupMode = WhichListItem("IV_2d_outwave", AllOutwaves)		
						
					DAC_PopupControl("DAC0_tab0",2,"MC current")
					ControlUpdate/W= Switchboard DAC0_tab0; PopupMenu DAC0_tab0 mode=2
					DACWaveSelect_Pop("dac0wave0_tab0",1,"IV_2d_outwave")	
					ControlUpdate/W=Switchboard dac0wave0_tab0; PopupMenu dac0wave0_tab0 mode=popupMode+1
					ADC_PopupControl("ADC0_tab1",3,"MC voltage")
					ControlUpdate/W= Switchboard ADC0_tab1; PopupMenu ADC0_tab1 mode=3
					ADC_PopupControl("ADC2_tab1",2,"MC current")
					ControlUpdate/W= Switchboard ADC2_tab1; PopupMenu ADC2_tab1 mode=2
					
					SaveMode=1
					ControlUpdate/W=Main SaveMode_tab0; PopupMenu SaveMode_tab0 win=Main, mode=2
							
					break			
				
				case "IclampSecond_tab3":
				
					SweepLength=1000
					SweepInterval=6
					TotalSweeps=56
					saveMode=1
					start=-50;step=100
					
					If(!WaveExists(IV_2d_outwave))
						Do
							MakePulse(50,1,100,0,5,start+(step*n),.1,"tempwv")
							PackageTo2d($"tempwv","IV_2d_outwave")
							n+=1
						While(n<6)
						n=0
								
					EndIf
					AllOutwaves = Sortlist(Wavelist("*_outwave", ";", "" ),";",4)
					//Print AllOutwaves
					popupMode = WhichListItem("IV_2d_outwave", AllOutwaves)		
						
					DAC_PopupControl("DAC1_tab0",2,"MC current")
					ControlUpdate/W= Switchboard DAC1_tab0; PopupMenu DAC1_tab0 mode=2
					DACWaveSelect_Pop("dac1wav1_tab0",1,"IV_2d_outwave")	
					ControlUpdate/W=Switchboard dac1wav1_tab0; PopupMenu dac1wav1_tab0 mode=popupMode+1
					ADC_PopupControl("ADC1_tab1",3,"MC voltage")
					ControlUpdate/W= Switchboard ADC1_tab1; PopupMenu ADC1_tab1 mode=3	
					ADC_PopupControl("ADC3_tab1",2,"MC current")
					ControlUpdate/W= Switchboard ADC3_tab1; PopupMenu ADC3_tab1 mode=2
					
					SaveMode=1
					ControlUpdate/W=Main SaveMode_tab0; PopupMenu SaveMode_tab0 win=Main, mode=2
					break
				
				case "Iclamp1_Vclamp2_tab3":
				
					SweepLength=1000
					SweepInterval=6
					TotalSweeps=10
					saveMode=1
					
					
					If(!WaveExists(zero_outwave))
						Make/O/N=500 zero_outwave
								
					EndIf
					AllOutwaves = Sortlist(Wavelist("*_outwave", ";", "" ),";",4)
					//Print AllOutwaves
					popupMode = WhichListItem("zero_outwave", AllOutwaves)		
						
					DAC_PopupControl("DAC0_tab0",2,"MC current")
					ControlUpdate/W= Switchboard DAC0_tab0; PopupMenu DAC0_tab0 mode=2
					DAC_PopupControl("DAC1_tab0",3,"MC voltage")
					ControlUpdate/W= Switchboard DAC1_tab0; PopupMenu DAC1_tab0 mode=3
														
					DACWaveSelect_Pop("dac1wav1_tab0",1,"zero_outwave")	
					ControlUpdate/W=Switchboard dac1wav1_tab0; PopupMenu dac1wav1_tab0 mode=popupMode+1
						
					ADC_PopupControl("ADC1_tab1",2,"MC current")
					ControlUpdate/W= Switchboard ADC1_tab1; PopupMenu ADC1_tab1 mode=2	
					ADC_PopupControl("ADC0_tab1",3,"MC voltage")
					ControlUpdate/W= Switchboard ADC0_tab1; PopupMenu ADC0_tab1 mode=3	
					
					SaveMode=1
					ControlUpdate/W=Main SaveMode_tab0; PopupMenu SaveMode_tab0 win=Main, mode=2
					break
						
				case "Vclamp1_Iclamp2_tab3":
				
					SweepLength=1000
					SweepInterval=6
					TotalSweeps=10
					saveMode=1
					
					
					If(!WaveExists(zero_outwave))
						Make/O/N=500 zero_outwave
								
					EndIf
					AllOutwaves = Sortlist(Wavelist("*_outwave", ";", "" ),";",4)
					//Print AllOutwaves
					popupMode = WhichListItem("zero_outwave", AllOutwaves)		
						
					DAC_PopupControl("DAC1_tab0",2,"MC current")
					ControlUpdate/W= Switchboard DAC1_tab0; PopupMenu DAC1_tab0 mode=2
					DAC_PopupControl("DAC0_tab0",3,"MC voltage")
					ControlUpdate/W= Switchboard DAC0_tab0; PopupMenu DAC0_tab0 mode=3
														
					DACWaveSelect_Pop("dac0wave0_tab0",1,"zero_outwave")	
					ControlUpdate/W=Switchboard dac0wave0_tab0; PopupMenu dac0wave0_tab0 mode=popupMode+1
						
					ADC_PopupControl("ADC0_tab1",2,"MC current")
					ControlUpdate/W= Switchboard ADC0_tab1; PopupMenu ADC0_tab1 mode=2	
					ADC_PopupControl("ADC1_tab1",3,"MC voltage")
					ControlUpdate/W= Switchboard ADC1_tab1; PopupMenu ADC1_tab1 mode=3	
					
					SaveMode=1
					ControlUpdate/W=Main SaveMode_tab0; PopupMenu SaveMode_tab0 win=Main, mode=2
					break
												
				default:
					break
					
			endswitch
			
			// click code here
			break
	endswitch

	return 0
End


Function DAC_PopupControl(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	//Print "!!!!!!!!!!!!"+ctrlName
	//Print "popNUM" + num2str(popNum)
	//Print "popStr"+popStr

	//Print num2str(popNum)
	//.....GLOBALS.....//			
	Wave DAC_checklist_wave = root:Globals:physiology:DAC_ADC_globals:DAC_checklist_wave
	

	SVAR DACgainString = root:Globals:physiology:DAC_ADC_globals:DACgainString
	SVAR DAC_string_preBuffer=root:Globals:physiology:DAC_ADC_globals:DAC_string_preBuffer
	NVAR TTL_isOn=root:Globals:physiology:DAC_ADC_globals:TTL_isOn
	//.........................// 
	
	//Variable last = strlen(ctrlName)-1
	Variable last=3
	//Variable current_DAC = str2num(ctrlName[last])
	Variable current_DAC = str2num(ctrlName[3])

	Variable j	

	//String match_string = "*"+ctrlName[last]+"*"
	String match_string = "*"+ctrlName[3]+"*"
	String tempString=""
		
	If (TTL_isON)		// If at least one TTL is on... 			
		DAC_string_preBuffer=ReplaceString("D",DAC_string_preBuffer,"")	// ..temporarily remove the "D"... we'll pop it back on later. 
	Endif
	
	If (popNum!=1)	// If the selected channel isn't OFF..
		DACgainString[str2num(ctrlName[last]),str2num(ctrlName[last])] = num2str(popNum-1) //.. set the gain and check it as "ON"
		DAC_checklist_wave[str2num(ctrlName[last])]=popNum-1

		If (!stringmatch(DAC_string_preBuffer,match_string))	// If the selected channel isn' already represented in the output string...
			DAC_string_preBuffer = DAC_string_preBuffer + ctrlName[last]	// .. then add it
		Else
			DAC_string_preBuffer = DAC_string_preBuffer
		Endif
	
	Else	// If the selected channel is OFF ..
		DAC_string_preBuffer = ReplaceString(ctrlName[last],DAC_string_preBuffer,"") // .. remove it from the list, and set the gain to 0
		DACgainString[str2num(ctrlName[last]),str2num(ctrlName[last])] ="0" 
		
		If(numpnts(DAC_checklist_wave)>0)	//.. and uncheck it from the checklist
			DAC_checklist_wave[str2num(ctrlName[last])]=0
		EndIf
	
	Endif
	
	// Sort the channels in ascending numerical order and pop on the terminal "D" if at least one TTL is on
	For(j=0;j<=2*strlen(DAC_string_preBuffer);j+=1)
		If(!mod(j,2)==0)
			tempString[j]=";"
		Else
			tempString[j]=DAC_string_preBuffer[j/2]
		Endif
	EndFor
	
	tempString=SortList(tempString,";",2)
	DAC_string_preBuffer = ReplaceString(";",tempString,"")
	
	If (TTL_isON)
		DAC_string_preBuffer=DAC_string_preBuffer+"D"
	Endif
	
	//	UpdateDAC_combo_list()		// PERHAPS BETTER TO DO THIS RIGHT BEFORE A SWEEP?
End
