#pragma rtGlobals=1		// Use modern global access method.
// Update: 6/26 

//////////////////////////////////////
// Make Globals for the Frame viewer
//////////////////////////////////////
Function MakeImageAnalysisGlobals()

	String dfSav = GetDataFolder(1)

	If(DataFolderExists("root:Globals:imaging:ImageAnalysisGlobals"))
		return 0
	Else
		NewDataFolder/O/S root:Globals:imaging:ImageAnalysisGlobals
		Make/O/N=1000 contrast_lookup; SetScale/I x,0,1, contrast_lookup
		Make/O/N=(512,512) FrameToShow
		Variable/G FrameViewerSpeed
		Variable/G gRadioVal
		Variable/G gRadioVal_range
		String/G showInViewer="*"
		String/G ImagesFolder =""
		String/G ImageStem=""
		String/G zapString
		String/G ColorTable
		String/G MovieOnDisplay
		String/G WorkingFolder
		String/G CurrentRangeMode
		Variable/G frameNumOfDisplayed
		Variable/G nzmax
		Variable/G nzmin
		Variable/G gamma_val=1
	EndIf
	
	SetDataFolder dfSav
End


//////////////////////////////////////
// Display the movie $inWaveString in destWindow @ rate of frameRate_ms
//////////////////////////////////////
Function ShowAsMovie(inWaveString,frameRate_ms,destWindow)
	String inWaveString
	Variable frameRate_ms
	String destWindow
	
	String destWindowMatch = WinList(destWindow,";","WIN:1")
	
	If(Strlen(destWindowMatch)==0)	// No such window?
		
		return 0
	EndIf

	If(!(WaveExists($inWaveString)))// No such wave, or not enough frames for movie
		
		return 0
	Else
		Wave inwave=$inWaveString
 		
		Variable last = dimsize(inwave,2)
	EndIf
	
	Variable settingsExist
	
	If(DataFolderExists("root:Globals:imaging:imageAnalysisGlobals"))
		settingsExist=1
	EndIf
	
	
	Variable dum_start

	SVAR MovieOnDisplay = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	SVAR/Z ColorTable = root:Globals:imaging:ImageAnalysisGlobals:ColorTable
	NVAR/Z nzmin= root:Globals:imaging:ImageAnalysisGlobals:nzmin
	NVAR/Z nzmax= root:Globals:imaging:ImageAnalysisGlobals:nzmax
	String contrastLookup = "root:Globals:imaging:ImageAnalysisGlobals:contrast_Lookup"

	If(dimsize(inwave,2)==0)
		return 0
	Else
		
		Variable frame_num=0
		UDFrameViewer(destWindow,inWaveString,frame_num,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
		
		If(settingsExist)		
			AppImageSettings(destWindow,contrastLookup,nzMin,nzMax,colorTable,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
		EndIf
				
	EndIf

	String OpenGraphs = WinList("FrameViewer",";","WIN:1")
	
		
	Do
		UDFrameViewer(destWindow,inWaveString,frame_num,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
		
		If(settingsExist)		
			AppImageSettings(destWindow,contrastLookup,nzMin,nzMax,colorTable,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
		EndIf
		
		dum_start=ticks	
		Do
			If(16.7*(ticks-dum_start)>frameRate_ms)	
				
				DoUpdate
				dum_start=ticks
				break
			EndIf
		While(1)
		
		frame_num+=1
		
	While (frame_num<last)	
	
	UDFrameViewer(destWindow,inWaveString,0,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
	
	If(settingsExist)		
		AppImageSettings(destWindow,contrastLookup,nzMin,nzMax,colorTable,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
	EndIf

	
	
End


//////////////////////////////////////
// Zap all images from the top graph
//////////////////////////////////////
Function RemoveAllImages()
	Variable index

	Do	
		String ImageList=ImageNameList("",";")
		Do
			String thisImage=StringFromList(index,ImageList)
			If(strlen(thisImage)==0)
				break
			EndIf
				
			RemoveImage/Z $thisImage
			index+=1
		
		While(1)
		index=0
	While(strlen(ImageList)!=0)
End




//////////////////////////////////////
// Calculates max/min range values 
// for image display using image statistics. 
//////////////////////////////////////
Function/S AutoRange(inwave,mode)
	Wave inwave
	String mode
	String histWave=NameOfWave(inwave)+"_hist"
	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	Wave/Z FrameToShow=root:Globals:imaging:ImageAnalysisGlobals:FrameToShow

	Make/O/N=100 $histWave

	Histogram/B=1 inwave, $histWave
	Wave histw=$histWave
	Variable tot=sum(histw,pnt2x(histw,0),pnt2x(histw,numpnts(histw)-1))

	Strswitch (mode)	
	
		case "94":

			///////// Code below is duplication from Wavemetrics ImageRangeAdjust.ipf  ////////
			variable s=0,i=0
			do
				s += histw[i]
				i+=1
			while( (s/tot) < 0.03 )
			nzmin= LeftX(histw)+deltax(histw)*i
	
			s=0;i=numpnts(histw)-1
			do
				s += histw[i]
				i-=1
			while( (s/tot) < 0.03 )
			nzmax= LeftX(histw)+deltax(histw)*i

			ModifyImage frameToShow ctab= {nzmin,nzmax,}
			break

		case "Auto":
			nzmax=Rightx(histw)
			nzmin=Leftx(histw)
			ModifyImage frameToShow ctab= {nzmin,nzmax,}// {*,*,}
			break
		
		case "2SD":
			Imagestats/Q frameToShow
			nzmax=V_avg+V_sdev/2
			nzmin = V_avg-2*V_sdev/2
			ModifyImage frameToShow ctab= {nzmin,nzmax,}// {*,*,}
			break
		
		case "DFF":
			nzmax=.1
			nzmin=-.01
			break
			
		default:
			break
	endswitch	
	
	return num2str(nzmin)+";"+num2str(nzmax)

End


//////////////////////////////////////
// Bumps the pixel range of a global wave up or down by 3%
//////////////////////////////////////
Function ChangeImageRange(direction)
	String Direction

	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	Wave FrameToShow  = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow

	strswitch(direction)
		case "Down":
			nzmax*=1.03
			nzmin*=1.03
			break

		case "Up":
			nzmax*=.97
			nzmin*=.97
			break

		default:
			break

	endswitch
	ModifyImage frameToShow ctab={nzmin,nzmax,}

End


//////////////////////////////////////
// 
//////////////////////////////////////
Function InvertContrast()
	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	Wave FrameToShow  = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow	
	Wave contrast_lookup=root:Globals:imaging:ImageAnalysisGlobals:contrast_lookup
	Redimension/N=(1000) contrast_lookup;SetScale/I x,0,1,contrast_lookup
	contrast_lookup=1-x		
	//ModifyImage frameToShow ctab={nzmin,nzmax,},lookup=contrast_lookup
	ModifyImage frameToShow lookup=contrast_lookup
End

//////////////////////////////////////
// 
//////////////////////////////////////
Function LinearContrast()
	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	Wave FrameToShow  = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow	
	Wave contrast_lookup = root:Globals:imaging:ImageAnalysisGlobals:contrast_lookup
	contrast_lookup=x
	ModifyImage frameToShow ctab={nzmin,nzmax,},lookup=contrast_lookup
End

//////////////////////////////////////
// 
//////////////////////////////////////
Function GammaContrast()
	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	Wave FrameToShow  = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow	
	NVAR gamma_val = root:Globals:imaging:ImageAnalysisGlobals:gamma_val
	Wave/Z contrast_lookup=root:Globals:imaging:ImageAnalysisGlobals:contrast_lookup
	Redimension/N=(1000) contrast_lookup;SetScale/I x,0,1,contrast_lookup
	
	contrast_lookup=x^(1/gamma_val)
	ModifyImage frameToShow ctab={nzmin,nzmax,}, lookup=contrast_lookup
	
End

//////////////////////////////////////
// Returns a list of folders under root:
//////////////////////////////////////
Function/S FolderList()	

	String dfSav = GetDataFolder(1)

	String FolderList = ""

	Variable totalFolders = CountObjects("root:",4)
	Variable index

	Do
		String thisFolder = GetindexedObjName("root:", 4, index)
		
		If(strlen(thisFolder)==0)
			break
		EndIf
		
		FolderList=FolderList+";"+thisFolder
		
		index+=1
		
	While(1)

	SetDataFolder dfSav
	
	Return FolderList
	
End


//////////////////////////////////////
//Puts a new 2D slice from a 3D movie into the named graph (destWindow)
//////////////////////////////////////
Function UDFrameViewer(destWindow,inWaveString,FrameNumber,outWaveName)
	String destWindow
	String inWaveString
	Variable FrameNumber

	String outWaveName // Name of the 1-D wave to be written

	////////// Test to make sure parameters are reasonable.... ////// 

	String destWindowMatch = WinList(destWindow,";","WIN:1")

	If(Strlen(destWindowMatch)==0)	// No such window?
		//Print "Busted1"
		return 0
	EndIf

	If(!(WaveExists($inWaveString)))	// No such wave
		//Print "Busted2"
		return 0
	Else
		Wave inwave=$inWaveString
	EndIf
 		

//	If(FrameNumber>dimsize($inWaveString,2)-1)
//		Print "Busted3"
//		return 0
//	EndIf
	
	
	Variable left_min,left_max,bot_min,bot_max
	
	GetAxis/Q/W=$destWindow left;left_min=V_min;left_max=V_max
	GetAxis/Q/W=$destWindow bottom; bot_min=V_min;bot_max=V_max
	
	RMImages(destWindow)
	
	If(dimsize(inwave,2)==1) 
		//Print "IS2d"
		Duplicate inwave $outWaveName
	Else
		//Print "ISNOT2d"
		Duplicate/O/R=[0,*][0,*][FrameNumber,FrameNumber] inWave $outWaveName
	EndIf
	
	AppendImage/W=$destWindow $outWaveName
	
	//SetAxis/W=$destWindow left left_min,left_max
	//SetAxis/W=$destWindow bottom bot_min,bot_max
	
End


//////////////////////////////////////
// Applies image settings (range, contrast, colortable) 
// to $theWave in a target graph (destWindow)
//////////////////////////////////////
Function AppImageSettings(destWindow,contrastLookup,nzMin,nzMax,colorTable,theWave)
	String destWindow
	String contrastLookup
	Variable nzMin,nzMax
	String colorTable
	String theWave

	String destWindowMatch = WinList(destWindow,";","WIN:1")

	If(Strlen(destWindowMatch)==0)	// No such window?	
		return 0
	EndIf
	
	String localWvName = NameOfWave($theWave)
		
	
	If(FindListItem(localWvName,ImageNameList(destWindow,";"))==-1)
	
		return 0
	Else

		ModifyImage/W=$destWindow $localWvName, ctab={nzMin,nzMax,$ColorTable},lookup=$contrastLookup
	EndIf
	
	
End

//////////////////////////////////////
// Removes all images from the graph destWindow
//////////////////////////////////////
Function RMImages(destWindow)
	String destWindow

	String destWindowMatch = WinList(destWindow,";","WIN:1")

	If(Strlen(destWindowMatch)==0)	// No such window?
		return 0
	EndIf

	Variable index

	Do	// Don't know why RemoveImage is problematic... this is a pretty dumb kludge
		String ImageList=ImageNameList(destWindow,";")
				
		Do
			String thisImage=StringFromList(index,ImageList)
			If(strlen(thisImage)==0)
				break
			EndIf
				
			RemoveImage/Z $thisImage
			index+=1
		
		While(1)
		
		index=0
	
	While(strlen(ImageList)!=0)

End

//////////////////////////////////////
//Calculates minima/maxima for image range
// for different settings.
//////////////////////////////////////
Function/S AR(inwave_string,mode)
	String inwave_string
	String mode
	String histWave=NameOfWave(inwave)+"_hist"
	
	Variable nzmin
	Variable nzmax
	
	If(!WaveExists($inwave_string))
		return ""
	EndIf
	
	Make/O/N=100 $histWave

	Histogram/B=1 $inwave_string, $histWave

	Wave histw=$histWave
	Variable tot=sum(histw,pnt2x(histw,0),pnt2x(histw,numpnts(histw)-1))

	Strswitch (mode)	
	
		case "ninety_four":

			///////// Code below is duplication from Wavemetrics ImageRangeAdjust.ipf  ////////
			variable s=0,i=0
			do
				s += histw[i]
				i+=1
			while( (s/tot) < 0.03 )
			nzmin= LeftX(histw)+deltax(histw)*i
	
			s=0;i=numpnts(histw)-1
			do
				s += histw[i]
				i-=1
			while( (s/tot) < 0.03 )
			nzmax= LeftX(histw)+deltax(histw)*i

			
			break

		case "Auto":
			
			nzmax=Rightx(histw)
			nzmin=Leftx(histw)
			
			break
		
		case "SD2":
			
			If(dimsize($inwave_string,1)>0)
				Imagestats/Q $inwave_string
				nzmax=V_avg+V_sdev/2
				nzmin = V_avg-2*V_sdev/2
			Else
				return ""
			EndIf
			
			break
		
		case "DFF":
			
			nzmax=.02
			nzmin=-.05
			break
			
		default:
			break
	endswitch	
	
	return num2str(nzmin)+";"+num2str(nzmax)

End

//////////////////////////////////////
// Sets the image range
//////////////////////////////////////
Function AdjustImageRange(destWindow, nzmin, nzmax,theWave) // nzmin,max are string args bcs. AR returns a string
	String destWindow
	String nzmin,nzmax
	String theWave
	
	String destWindowMatch = WinList(destWindow,";","WIN:1")

	If(Strlen(nzmin)==0 || Strlen(nzmax)==0)
		return 0
	EndIf
	
	If(Strlen(destWindowMatch)==0)	// No such window?	
		return 0
	EndIf
	
	String localWvName = NameOfWave($theWave)
		
	
	If(FindListItem(localWvName,ImageNameList(destWindow,";"))==-1)
	
		return 0
	Else

		ModifyImage/W=$destWindow $localWvName, ctab={str2num(nzmin),str2num(nzmax),}
	
	EndIf


End



///////////////////////////////////////////////// PANELS AND BUTTONS ///////////////////////////////////////////
///////////////////////////////////////////////// PANELS AND BUTTONS ///////////////////////////////////////////
///////////////////////////////////////////////// PANELS AND BUTTONS ///////////////////////////////////////////


///////////////////////////////////// BUTTONS

//////////////////////////////////////
// Make the top Image brighter or darker
//////////////////////////////////////
Function BrightnessButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			If(StringMatch(ba.ctrlName,"Brighter"))
				ChangeImageRange("Up")
			ElseIf (StringMatch(ba.ctrlName,"Darker"))
				ChangeImageRange("Down")
			EndIf
			
			break
	endswitch

	return 0
End

//////////////////////////////////////
// Play 3d wave as a movie
//////////////////////////////////////
Function PlayMovie_Button(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	NVAR FrameRate_ms = root:Globals:imaging:ImageAnalysisGlobals:FrameViewerSpeed
	SVAR ExperimentName = root:Globals:physiology:Run_globals:ExperimentName
	SVAR Movie = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	
	String MovieOnDisplay = "root:"+ExperimentName+":imaging_data:"+Movie
		
	switch( ba.eventCode )
		case 2: // mouse up
			ShowAsMovie(MovieOnDisplay,frameRate_ms,"FrameViewer_subWin")	
	endswitch

	return 0
End

//////////////////////////////////////
// 
//////////////////////////////////////
Function FrameForward_button(ba) : ButtonControl

	STRUCT WMButtonAction &ba

	If(!DataFolderExists("root:Globals:imaging:ImageAnalysisGlobals:"))
		return 0
	EndIf
	
	// Globals //
	NVAR FrameNumOfDisplayed= root:Globals:imaging:ImageAnalysisGlobals:FrameNumOfDisplayed
	NVAR nzmin= root:Globals:imaging:ImageAnalysisGlobals:nzmin
	NVAR nzmax=root:Globals:imaging:ImageAnalysisGlobals:nzmax

	SVAR Movie = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	SVAR ColorTable = root:Globals:imaging:ImageAnalysisGlobals:ColorTable	
	SVAR WorkingFolder = root:Globals:imaging:ImageAnalysisGlobals:WorkingFolder
	// ..........//
	
	String MovieOnDisplay = WorkingFolder+":imaging_data:"+Movie
	
	If(!WaveExists($MovieOnDisplay))
		return 0
	Else
		Wave mv_display = $MovieOnDisplay
	EndIf
	
	//Variable left_min,left_max,bot_min,bot_max
	
	String contrast_lookup ="root:Globals:imaging:ImageAnalysisGlobals:contrast_lookup"
		
	switch( ba.eventCode )
		case 2: // mouse down
		
			strswitch(ba.ctrlName)
			
				Case "forward":
					If(FrameNumOfDisplayed<(dimsize($MovieOnDisplay,2)-1))
						FrameNumOfDisplayed+=1
				
					ElseIf(FrameNumOfDisplayed>dimsize($MovieOnDisplay,2)-1)
						FrameNumOfDisplayed=dimsize($MovieOnDisplay,2)-1
					
					EndIf
				
					break
						
				Case "backward":
					If(FrameNumOfDisplayed>0)
						FrameNumOfDisplayed-=1
					EndIf
				
					break
			
				default:
					break
				
			endswitch 
			break
	
	endswitch
	
	UDFrameViewer("FrameViewer_subWin",MovieOnDisplay,FrameNumOfDisplayed,"root:Globals:imaging:ImageAnalysisGlobals:FrameToShow")
	AppImageSettings("FrameViewer_subWin",contrast_lookup,nzmin,nzmax,ColorTable,"root:Globals:imaging:ImageAnalysisGlobals:FrameToShow")
	
	return 0

End

Function AutoRange_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	SVAR Movie = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay	
	SVAR WorkingFolder = root:Globals:imaging:ImageAnalysisGlobals:WorkingFolder
	SVAR currentRangeMode = root:Globals:imaging:ImageAnalysisGlobals:currentRangeMode
	
	String inwave_string = WorkingFolder+":imaging_data:"+ Movie
	String minval,maxval	
	String outwaveString = "root:Globals:imaging:ImageAnalysisGlobals:FrameToShow"
	
	If(!WaveExists($inwave_string))
		return 0
	EndIf

	String mode = ""
	
	switch( ba.eventCode )
		case 2: // mouse up
		
			mode=ba.ctrlName
			String range_vals = AR(inwave_string,mode)
			
			If(strlen(range_vals)==0)
				return 0
				
			Else
				minval=StringFromList(0,range_vals,";")
				maxval = StringFromList(1,range_vals,";")
				
				nzmin=str2num(minval)
				nzmax=str2num(maxval)
				
			EndIf
		
	
	
			If(!WaveExists($outwaveString))
				
				return 0
			Else
	
				AdjustImageRange("FrameViewer_subwin", minval, maxval, outwaveString)
				currentRangeMode = mode
			EndIf
			break
	endswitch
	
	
	
	return 0
End


///////////////////////////////////// PopUp Menus


//////////////////////////////////////
// Load a new image into the frame viewer
//////////////////////////////////////
Function NewImage_Popup(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	
	SVAR MovieOnDisplay=root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	SVAR ColorTable = root:Globals:imaging:ImageAnalysisGlobals:ColorTable
	NVAR FrameNumOfDisplayed= root:Globals:imaging:ImageAnalysisGlobals:FrameNumOfDisplayed
	
	SVAR WorkingFolder = root:Globals:imaging:ImageAnalysisGlobals:WorkingFolder
	SVAR mode = root:Globals:imaging:ImageAnalysisGlobals:currentRangeMode
	String contrast_lookup ="root:Globals:imaging:ImageAnalysisGlobals:contrast_lookup"
			
	NVAR nzMin = root:Globals:imaging:ImageAnalysisGlobals:nzMin
	NVAR nzMax = root:Globals:imaging:ImageAnalysisGlobals:nzMax
	
	
	FrameNumOfDisplayed=0
	
	
	Variable index
	
	RMIMages("FrameViewer_subwin")
	
	
	switch( pa.eventCode )
		
		case 2: // mouse up
			
			Variable popNum = pa.popNum
			
			MovieOnDisplay = pa.popStr
			
			String FullWavePath = WorkingFolder + ":imaging_data:"+MovieOnDisplay
				
			If(!WaveExists($FullWavePath))
				return 0
			Else
				
			
			
				String RangeVals =  AR(FullWavePath,"Auto")
				String minval=StringFromList(0,RangeVals,";"); nzmin=str2num(minval)
				String maxval=StringFromList(1,RangeVals,";");nzmax=str2num(maxval)
				
				
				UDFrameViewer("FrameViewer_subWin",FullWavePath,FrameNumOfDisplayed,"root:Globals:imaging:ImageAnalysisGlobals:FrameToShow")
			
				AppImageSettings("FrameViewer_subWin",contrast_lookup,nzMin,nzMax,colorTable,"root:Globals:imaging:ImageAnalysisGlobals:FrameToShow")
			
			//	SetAxis/A/W=FrameViewer_subwin left
			//	SetAxis/A/W=FrameViewer_subwin bottom
			
			EndIf
			

			break	
	endswitch

	return 0
End


//////////////////////////////////////
// Adjust Contrast of the top graph
//////////////////////////////////////
Function Contrast_popup(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	ControlInfo gamma_val
	
			
	switch( pa.eventCode )
		case 2: // mouse up
			strswitch(pa.popStr)
			
				case "Linear":
					LinearContrast()
					V_disable = V_disable | 0x1
					//ModifyControl "gammaVal" 
					break
			
				case "Invert":
					InvertContrast()
					V_disable = V_disable | 0x1
					break
			
				case "Gamma":
					GammaContrast()
					V_disable = V_disable & ~0x1
					break
				
				default:
					break
				
			endswitch
			
			Variable popNum = pa.popNum
			String popStr = pa.popStr
			break
	endswitch
	
	
	ModifyControl/Z  Gamma_val, win=FrameViewer_subWin#ImageControls, disable=V_disable
	
	return 0
End


//////////////////////////////////////
// Sets the working folder from which images are chosen
//////////////////////////////////////
Function DataFolderList_popup(pa) : PopupMenuControl

	STRUCT WMPopupAction &pa
	
	SVAR WorkingFolder = root:Globals:Imaging:ImageAnalysisGlobals:WorkingFolder
	
	switch( pa.eventCode )
		case 2: // mouse up
			Variable popNum = pa.popNum
			String popStr = pa.popStr
			break
	endswitch

	WorkingFolder = "root:" + popStr
	
	return 0
End


//////////////////////////////////////
// Sets the colortable for the top image
//////////////////////////////////////
Function ColorTablePicker_popUp(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	SVAR MovieOnDisplay = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	SVAR ColorTable=root:Globals:imaging:ImageAnalysisGlobals:ColorTable
	
	Wave/Z FrameToShow = root:Globals:imaging:ImageAnalysisGlobals:FrameToShow
	
	switch( pa.eventCode )
		case 2: // mouse up
			Variable popNum = pa.popNum
			String popStr = pa.popStr
			ColorTable = popStr
			
			ModifyImage FrameToShow ctab= {*,*,$popStr,0} 
			break
	endswitch

	return 0
End



///////////////////////////////////// SETVARS

//////////////////////////////////////
// Sets gamma -value for contrast and auto-updates
//////////////////////////////////////
Function Gamma_setVar(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	switch( sva.eventCode )
	
		case 2: // Enter key
		
			ControlInfo ContrastPicker
			If(Stringmatch(S_value,"Gamma"))
			//	Print "SVAL+" + S_value
				GammaContrast()
			EndIf						
		
	endswitch

	return 0
End


//////////////////////////////////////
// Sets the current frame being displayed
//////////////////////////////////////
Function SetFrame_sva(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	
	NVAR frameNumOfDisplayed = root:Globals:imaging:ImageAnalysisGlobals:frameNumOfDisplayed
	NVAR nzmax = root:Globals:imaging:ImageAnalysisGlobals:nzmax
	NVAR nzmin = root:Globals:imaging:ImageAnalysisGlobals:nzmin
	SVAR ColorTable = root:Globals:imaging:ImageAnalysisGlobals:ColorTable
	SVAR MovieOnDisplay = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	SVAR WorkingFolder = root:Globals:imaging:ImageAnalysisGlobals:WorkingFolder
	String contrast_lookup="root:Globals:imaging:ImageAnalysisGlobals:contrast_lookup"
	
	
	String inwave_string = WorkingFolder+":imaging_data:"+MovieOnDisplay
	
	
	If(!(WaveExists($inwave_string)))
		return 0
	Else
		Wave ww  = $inwave_string
	EndIf
	
	
	Wave/Z frameToShow = root:Globals:imaging:ImageAnalysisGlobals:frameToShow
	
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			
			If ((frameNumOfDisplayed>=0) && (frameNumOfDisplayed <(dimsize(ww,2))))
		
				UDFrameViewer("FrameViewer_subWin",inWave_String,frameNumOfDisplayed,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
				AppImageSettings("FrameViewer_subWin",contrast_lookup,nzmin,nzmax,ColorTable,"root:Globals:imaging:ImageAnalysisGlobals:FrameToShow")
			
			ElseIf (frameNumOfDisplayed > dimsize(ww,2))
				frameNumOfDisplayed=dimsize(ww,2)-1
				UDFrameViewer("FrameViewer_subWin",inWave_String,frameNumOfDisplayed,"root:Globals:imaging:ImageAnalysisGlobals:frameToShow")
				AppImageSettings("FrameViewer_subWin",contrast_lookup,nzmin,nzmax,ColorTable,"root:Globals:imaging:ImageAnalysisGlobals:FrameToShow")
				
			EndIf
								
		case 3: // Live update
			Variable dval = sva.dval
			String sval = sva.sval
			break
	endswitch

	return 0
End

//////////////////////////////////////
// Sets the type of images to be displayed (DFF, no DFF, etc...)
// when user enters data into wave_stem (in Frame viewer)
//////////////////////////////////////
Function SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	NVAR gRadioVal = root:Globals:imaging:ImageAnalysisGlobals:gRadioVal
	
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			Switch (gRadioVal)
				
				case 1:
					MyCheckProc("All",1)
					break
				case 2:
					MyCheckProc("DFFonly",2)
					break
				case 3: 
					MyCheckProc("NoDFF",3)
					break
					
			EndSwitch		
		
			break
	endswitch

	return 0
End


///////////////////////////////////// CHECKBOX

//////////////////////////////////////
// Radio Button to set which type of images to show (DFF, non-DFF..) 
//////////////////////////////////////
Function MyCheckProc(name,value):CheckBoxControl
	String name
	Variable value
	
	
	NVAR gRadioVal= root:Globals:imaging:ImageAnalysisGlobals:gRadioVal
	SVAR showInViewer=root:Globals:imaging:ImageAnalysisGlobals:showInViewer
	SVAR ImageStem = root:Globals:imaging:ImageAnalysisGlobals:ImageStem
	SVAR zapString = root:Globals:imaging:ImageAnalysisGlobals:zapString
	SVAR ExperimentName = root:Globals:physiology:Run_globals:ExperimentName
	SVAR WorkingFolder = root:Globals:imaging:ImageAnalysisGlobals:WorkingFolder
	
	String dfSav = GetDataFolder(1)
	
	String PathToExperiment = "root:"+ExperimentName+":imaging_data"
	
	String fldrName = WorkingFolder+":imaging_data"
	

	If(DataFolderExists(PathToExperiment))
		SetDataFolder fldrName
		Variable IsSetUp=1
	EndIf
	
	
	String GrepTest 
	
	Strswitch(name)
		case "All":
			gRadioVal= 1
			
			If (IsSetUp)
				showInViewer=WaveList(ImageStem+"*",";","MINCOLS:1")
			Else
				showInViewer=""
			EndIf
				
			break
		
		case "DFFonly":
			gRadioVal= 2 
			
			If (IsSetUp)
				showInViewer=WaveList(ImageStem+"*_DFF",";","MINCOLS:2")
			Else
				showInViewer=""
			EndIf
				
			break
		
		case "NoDFF":
			gRadioVal= 3
			
			If (IsSetUp)
				String StringToGrep = WaveList(ImageStem+"*",";","MINCOLS:1")
				showInViewer=GrepList(StringToGrep,"_DFF",1)
			Else
				showInViewer=""
			EndIf
			break
	endswitch
	
	CheckBox All,Win=FrameViewer_subWin#ImageControls,value= gRadioVal==1
	CheckBox DFFonly,Win=FrameViewer_subWin#ImageControls,value= gRadioVal==2
	CheckBox NoDFF,Win=FrameViewer_subWin#ImageControls,value= gRadioVal==3
	
	SetDataFolder dfSav
End


//////////////////////////////////////
// The main Frame viewer Panel
//////////////////////////////////////
Function AcqMakeFrameViewer()

	DoWindow/F FrameViewer_subWin
	If(V_Flag==1)
		return 0
	EndIf
		
	InitializeExperiment()
	
	PauseUpdate; Silent 1		// building window...
	Display/N=FrameViewer_subWin /W=(643.5,170.75,1033.5,517.25) as "Frame Viewer"
	AppendImage :Globals:imaging:ImageAnalysisGlobals:FrameToShow
	ModifyImage FrameToShow ctab= {1905,2583,Grays,0}
	ModifyImage FrameToShow lookup= :Globals:imaging:ImageAnalysisGlobals:contrast_lookup
	ModifyGraph mirror=2
	SetAxis left -0.5,511.5
	SetAxis bottom -0.5,511.5
	SetDrawLayer UserFront
	SetDrawEnv translate= 0.473529,-0.088586,rotate= -11.841,rsabout
	SetDrawEnv save
	SetDrawEnv save
	NewPanel/HOST=#/EXT=2/W=(0,0,287,68) 
	ModifyPanel fixedSize=0
	GroupBox group0,pos={4,7},size={277,49},labelBack=(52224,52224,52224)
	GroupBox group0,fColor=(52224,52224,52224)
	Button button5,pos={8,14},size={50,34},proc=DFF_QuickTab,title="DF/F"
	Button button5,fColor=(13056,0,0)
	Button button6,pos={61,14},size={50,34},proc=ROI_QuickTab,title="ROI"
	Button button6,labelBack=(0,3072,13056),fColor=(0,6400,26112)
	Button button7,pos={116,14},size={50,34},title="LOI",fColor=(6400,13056,0)
	Button button8,pos={170,14},size={50,34},title="Movie",fColor=(13056,0,26112)
	Button button9,pos={225,14},size={50,34},title="Plotter"
	RenameWindow #,QuickTabs
	SetActiveSubwindow ##
	NewPanel/HOST=#/EXT=3/W=(0,99,560,0) 
	ModifyPanel fixedSize=0
	SetDrawLayer ProgBack
	SetDrawEnv linethick= 1.5,linefgc= (65280,65535,65535),fillpat= 0
	DrawRect 118.104651162791,161,155.882428940568,122
	DrawText 118.104651162791,162,"ROI"
	SetDrawLayer UserBack
	SetDrawEnv linefgc= (34816,34816,34816),fillpat= 0
	DrawRect 97,36,243,59
	SetDrawEnv linefgc= (34816,34816,34816),fillpat= 0
	DrawRect 97,8,243,31
	GroupBox group2,pos={3,2},size={75,45},labelBack=(52224,52224,52224)
	GroupBox group2,fColor=(65280,0,0)
	CheckBox DFFonly,pos={7,19},size={65,14},proc=MyCheckProc,title="\\F'Symbol'D\\F'Arial' F/F only"
	CheckBox DFFonly,fSize=10,value= 0,mode=1
	CheckBox NoDff,pos={7,33},size={55,14},proc=MyCheckProc,title="\\F'Arial'No \\F'Symbol'D\\F'Arial'F/F"
	CheckBox NoDff,fSize=10,value= 0,mode=1
	CheckBox All,pos={7,4},size={68,14},proc=MyCheckProc,title="All images"
	CheckBox All,font="Arial",fSize=10,value= 1,mode=1
	GroupBox group3,pos={87,2},size={164,91},labelBack=(52224,52224,52224)
	GroupBox group3,fColor=(65535,65535,65535)
	SetVariable setvar1,pos={100,38},size={110,20},proc=SetVarProc,title="Stem"
	SetVariable setvar1,fSize=13
	SetVariable setvar1,value= root:Globals:imaging:ImageAnalysisGlobals:ImageStem
	PopupMenu popup2,pos={221,10},size={21,21},proc=NewImage_Popup
	PopupMenu popup2,mode=0,value= #"root:Globals:imaging:ImageAnalysisGlobals:showInViewer"
	TitleBox title1,pos={103,14},size={15,13},fSize=10,frame=0
	TitleBox title1,variable= root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
	PopupMenu popup0,pos={98,64},size={147,21},proc=ColorTablePicker_popUp
	PopupMenu popup0,mode=1,bodyWidth= 147,popvalue="",value= #"\"*COLORTABLEPOP*\""
	GroupBox group6,pos={258,4},size={68,91},labelBack=(52224,52224,52224)
	GroupBox group6,fColor=(65535,65535,65535)
	Button forward,pos={295,43},size={27,16},proc=FrameForward_button,title=">>"
	Button forward,fSize=16,fStyle=1
	Button backward,pos={263,43},size={27,16},proc=FrameForward_button,title="<<"
	Button backward,fSize=16,fStyle=1
	SetVariable FrameNum,pos={274,23},size={35,20},proc=SetFrame_sva,title=" "
	SetVariable FrameNum,fSize=14,format="%g"
	SetVariable FrameNum,limits={0,inf,0},value= root:Globals:imaging:ImageAnalysisGlobals:frameNumOfDisplayed
	TitleBox title,pos={270,7},size={44,16},title="Frame",fSize=15,frame=0,fStyle=1
	TitleBox title,fColor=(30464,30464,30464)
	GroupBox group7,pos={4,50},size={75,43},labelBack=(52224,52224,52224)
	GroupBox group7,fColor=(65280,0,0)
	Button Play_Movie1,pos={13,73},size={57,18},proc=PlayMovie_Button,title="Play"
	Button Play_Movie1,labelBack=(52224,52224,52224),fSize=12
	Button Play_Movie1,fColor=(65535,65535,65535)
	Slider MovieSpeed1,pos={13,52},size={57,21}
	Slider MovieSpeed1,limits={100,10,1},variable= root:Globals:imaging:ImageAnalysisGlobals:FrameViewerSpeed,side= 0,vert= 0,ticks= 0,thumbColor= (0,15872,65280)
	GroupBox group0,pos={333,41},size={224,55},labelBack=(52224,52224,52224)
	SetVariable Max2,pos={393,43},size={94,16},title="Max",fSize=9
	SetVariable Max2,limits={0,inf,0},value= root:Globals:imaging:ImageAnalysisGlobals:nzmax,bodyWidth= 70
	SetVariable Max3,pos={393,65},size={93,16},title="Min",fSize=9
	SetVariable Max3,limits={-2,inf,0},value= root:Globals:imaging:ImageAnalysisGlobals:nzmin,bodyWidth= 72
	Button Brighter,pos={340,44},size={28,16},proc=BrightnessButtonProc,title="\\W617"
	Button Brighter,labelBack=(52224,52224,52224),fSize=10
	Button Darker,pos={340,65},size={28,16},proc=BrightnessButtonProc,title="\\W623"
	Button Darker,labelBack=(52224,52224,52224),fSize=10
	GroupBox group1,pos={333,2},size={225,34},labelBack=(52224,52224,52224)
	PopupMenu ContrastPicker,pos={400,9},size={73,21},proc=Contrast_popup,fSize=8
	PopupMenu ContrastPicker,mode=3,popvalue="Gamma",value= #"\"Linear;Invert;Gamma\""
	TitleBox title3,pos={335,11},size={57,16},title="Contrast",fSize=15,frame=0
	TitleBox title3,fStyle=1,fColor=(30464,30464,30464)
	SetVariable Gamma_val,pos={477,13},size={79,16},proc=Gamma_setVar,title="gamma"
	SetVariable Gamma_val,limits={-inf,inf,0},value= root:Globals:imaging:ImageAnalysisGlobals:gamma_val
	PopupMenu popup1,pos={261,65},size={63,21},proc=DataFolderList_popup,title="folder"
	PopupMenu popup1,mode=0,value= #"folderList()"
	Button Auto,pos={509,43},size={35,16},proc=AutoRange_button,title="Auto"
	Button Auto,labelBack=(52224,52224,52224)
	Button ninety_four,pos={509,60},size={35,16},proc=AutoRange_button,title="94%"
	Button DFF,pos={509,77},size={35,16},proc=AutoRange_button,title="DFF"
	RenameWindow #,ImageControls
	SetActiveSubwindow ##
End




//////////////////////////////////////////// WROTE THESE FOR AN EARLIER VERSION////////////////////////
//////////////////////////////////////////// WROTE THESE FOR AN EARLIER VERSION////////////////////////
//////////////////////////////////////////// WROTE THESE FOR AN EARLIER VERSION////////////////////////
///// Will be useful for later procs which do online analysis, etc./////////////////////////////////////////////////



//
//Function QuickCalc(inwave)
//	Wave inwave
//	Variable index
//
//	make/o/n=(dimsize(inwave,2)) ootwave
//
//	Do
//
//		ImageStats/P=(index) inwave
//		ootwave[index]=V_avg
//		index+=1
//	While(index<dimsize(inwave,2))
//
//End



////////////////// THis needs to write  to a folder /////////////////////
//Function PlaceImageProfileLine(GraphName)
//	String GraphName
//	Variable index
//
//	Variable leftMost,rightMost,topMost,BottomMost
//	Variable XCoord,YCoord_top,YCoord_bottom
//	
//	String ImagesGraphed=ImageNameList(GraphName,";")
//	
//	
//	If (ItemsInList(ImagesGraphed,";")!=1)
//		return 0
//	EndIf
//	
//	String WorkingAxes=AxisList(GraphName)
//			
//	Do
//		String thisAxis =StringFromList(index,WorkingAxes,";")
//			
//		If (strlen(thisAxis)==0)
//			break
//		EndIf
//			
//			
//		StrSwitch (thisAxis)
//			
//			case "top":
//				
//				GetAxis/Q/W=$GraphName top
//				XCoord=round(V_min+(V_Max-V_min)/2)
//				
//				break
//			
//			case "bottom":
//				GetAxis/Q/W=$GraphName bottom
//				XCoord=round(V_min+(V_Max-V_min)/2)
//				
//				break
//			
//			case "left":
//				
//				GetAxis/Q/W=$GraphName left
//				YCoord_top=ceil(V_min+(V_Max-V_min)*.1)
//				YCoord_bottom=floor(V_max-(V_Max-V_min)*.1)
//				
//				break
//			
//			case "right":
//				
//				GetAxis/Q/W=$GraphName right
//				YCoord_top=ceil(V_min+(V_Max-V_min)*.1)
//				YCoord_bottom=floor(V_max-(V_Max-V_min)*.1)
//				
//				break
//			
//			default:
//				break
//			
//		EndSwitch
//				
//		index+=1
//		
//	While(1)
//		
//	Print  num2str(XCoord)
//		
//	Make/O/N=2 X_coords={XCoord,XCoord}
//	Make/O/N=2 Y_coords={YCoord_top,YCoord_bottom}
//		
//	String axNames= StringByKey("AXISFLAGS",ImageInfo(GraphName,StringFromList(0,ImagesGraphed,";"),0))+" "
//		
//	String Traces1 = TraceNameList(GraphName,";",1)
//		
//	Execute "AppendToGraph"+axNames+ "Y_coords vs X_coords"
//		
//	String Traces2 = TraceNameList(GraphName,";",1)
//	Traces2=ReplaceString(";",RemoveFromList(Traces1,Traces2),"")
//			
//	Print Traces2
//	GraphWaveEdit $Traces2
//
//End



////////////////// THis needs to write stuff to a folder /////////////////////
//Function DFFAlongLoi(inwave,LOI_X,LOI_Y,firstBackground,lastBackground,[LOI_width])
//	Wave inwave
//	Wave LOI_X,LOI_Y
//	Variable firstBackground,lastBackground
//	Variable LOI_width
//
//	If(ParamIsDefault(LOI_width))
//		LOI_width=1
//	EndIf
//	
//	Make/O W_ImageLineProfile,W_LineProfileX,W_sumRows 	
//	Variable index
//
//	If(Dimsize(inwave,2)==0)
//		return 0
//	EndIf
//
//	ImageLineProfile/P=0 xWave=LOI_X,yWave=LOI_Y,width=LOI_width,srcwave=inwave
//	Make/O/N=(dimsize(W_LineProfileX,0),dimsize(inwave,2)) Line_DFF
//	Line_DFF[0,*][0]=W_ImageLineProfile[p]
//	index+=1
//		
//	Do
//		ImageLineProfile/P=(index) xWave=LOI_X,yWave=LOI_Y,width=LOI_width,srcwave=inwave
//		Line_DFF[0,*][index]=W_ImageLineProfile[p]
//	
//		index+=1
//	While(index<dimsize(inwave,2)) 
//
//	
//	Duplicate/O Line_DFF Line_outwave_DFF
//	Duplicate/O/R=[0,*][firstBackground,lastBackground] Line_DFF baseWave
//	ImageTransform sumAllRows baseWave
//	W_sumRows/=(lastBackground-firstBackground+1)
//	Line_DFF[0,*][0,*]=(Line_DFF[p][q]-W_sumRows[p])/W_sumRows[p] //W_sumRows[p]
//		
//End

/////////////////////// Function Show as Movie///////////////