#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.0

// Last modified: 6/26/08 (JBC)

//////////////////////////////////////
// Makes Globals for performing DFF on waves
//////////////////////////////////////
Function MakeDFFGlobals()

	String dfSav = GetDataFolder(1)

	If(DataFolderExists("root:Globals:imaging:ImageAnalysisGlobals:DFFTab"))
		return 0
	Else
		NewDataFolder/O/S root:Globals:imaging:ImageAnalysisGlobals:DFFTab
	EndIf
		Make/O/T DFF_seriesList
		Make/O DFF_selWave
		Variable/G firstBackground=0
		Variable/G lastBackground=4
		Variable/G DFFTab_seriesCount
		String/G DFF_tabStem	///// rig this up so its the same as the frame viewer stem by default ..... 
		String/G DFFCurrentMovie
		String/G ShowInDFFTab
		
	SetDataFolder dfSav
End


//////////////////////////////////////
// Populates the wavelist of viable 3D waves 
// to perform DF/F on 
//////////////////////////////////////

Function AvailForDFF()
SVAR DFF_tabStem = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_tabStem
SVAR SHowInDFFTab=root:Globals:imaging:ImageAnalysisGlobals:DFFTab:ShowInDFFTab
NVAR DFFTab_seriesCount = 	root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFFTab_seriesCount
SVAR ExperimentName = root:Globals:physiology:Run_globals:ExperimentName

String pathToImage = "root:"+ExperimentName+":imaging_data"

SetDataFolder pathToImage

	String tmpShow = WaveList(DFF_tabStem+"*",";","DIMS:3")	
	ShowInDFFTab = GrepList(tmpShow,"_DFF",1)
		DFFTab_seriesCount = ItemsInList(ShowInDFFTab,";")
		StringToTextWave(ShowInDFFTab)
	
End

//////////////////////////////////////
// Takes a list of Wave names and makes a 1D text
// wave out of them. Each row of the text wave is a wave
// name from the list
//////////////////////////////////////

Function StringToTextWave(inString)
String inString 
Variable index
Wave/T/Z DFF_seriesList = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_seriesList
Wave/Z DFF_selWave = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_selWave

	Variable itemsInString = ItemsInList(inString,";")
	Redimension/N=(itemsInString) DFF_seriesList,DFF_selWave
	
	Do
		String thisString=StringFromList(index,inString)
		
		If(Strlen(thisString)==0)
			break
		EndIf
		
		DFF_seriesList[index]=thisString
		
		index+=1
		
	While(1)		


End

	
//////////////////////////////////////
// Performs deltaF/F on the 3D wave$inString. 
// FirstBackground and LastBackground are
// variable setting the range for the baseline
// period of the calculation
//////////////////////////////////////
Function DFFMovie(inString,firstBackground,lastBackground)
	String inString
	Variable firstBackground,lastBackground
	Variable index

	String outWaveName = inString + "_DFF"

	If((Dimsize($inString,2)==0) || (!WaveExists($inString)))
		return 0
	EndIf
	
	Duplicate/O/R=[0,*][0,*][firstBackground,lastBackground] $inString base_image
	
	Make/O M_AveImage,M_ImagePlane
	Make/O/N=(dimsize($inString,0),dimsize($inString,1),dimsize($inString,2)) $outWaveName
	Wave ww = $outWaveName
	
	ImageTransform averageImage base_image	
	
	Do
		ImageTransform/P= (index) getPlane $inString
		MatrixOP/O DDest=(M_ImagePlane-M_AveImage)/(M_AveImage)
		ww[0,*][0,*][index]=DDest[p][q]	
		index+=1
	While(index<dimsize($inString,2))
	
	KillWaves/Z M_imagePlane,M_Ave_image
End 

//////////////////////////////////////
// Performs DF/F on each 3D wave in inwaveList
//////////////////////////////////////
Function DFFMovieList(inwaveList,selWave,firstBackground,lastBackground)
Wave/T inwaveList
Wave selWave
Variable firstBackground,lastBackground
Variable index

If(dimsize(inwaveList,0)!=dimsize(selWave,0))
	return 0
EndIf

	Do
		String thisWave = inwaveList[index]
			
			If(Strlen(thisWave)==0 || index>dimsize(inwaveList,0))
				break
			EndIf
			
			If(WaveExists($thisWave) && (selWave[index]!=0))
				DFFMovie(thisWave,firstBackground,lastBackground)
					
			EndIf
			
			
			index+=1
	While(1)	

End


////////////////////////////////////////////// BUTTONS AND PANELS ////////////////////////////////////////////
////////////////////////////////////////////// BUTTONS AND PANELS ////////////////////////////////////////////
////////////////////////////////////////////// BUTTONS AND PANELS ////////////////////////////////////////////


Function DFFStemProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			AvailForDFF()
		case 3: // Live update
			Variable dval = sva.dval
			String sval = sva.sval
			break
	endswitch

	return 0
End

Function DFFPick_popup(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	SVAR DFFCurrentMovie = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFFCurrentMovie
	
	switch( pa.eventCode )
		case 2: // mouse up
			Variable popNum = pa.popNum
			String popStr = pa.popStr
			DFFCurrentMovie = pa.popStr
			break
	endswitch

	return 0
End


Function SelectAllDFF_button(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	Wave DFF_selWave = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_selWave
	
	switch( ba.eventCode )
		case 2: // mouse up
			DFF_selWave=1
			break
	endswitch

	return 0
End


Function DFF_single_Button(ba) : ButtonControl
	STRUCT WMButtonAction &ba
		
	SVAR DFFCurrentMovie = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFFCurrentMovie
	NVAR firstBackground = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:firstBackground
	NVAR lastBackground=root:Globals:imaging:ImageAnalysisGlobals:DFFTab:lastBackground
	NVAR gRadioVal = root:Globals:imaging:ImageAnalysisGlobals:gRadioVal
	
	switch( ba.eventCode )
		case 2: // mouse up
				DFFMovie(DFFCurrentMovie,firstBackground,lastBackground)
			break
	endswitch
	
	switch(gRadioVal)
		case 1:
			MyCheckProc("All",1)
			break
		case 2:
			MyCheckProc("DFFonly",2)
			break
		case 3: 
			MyCheckProc("NoDFF",3)
			break
					
	EndSwitch	

	return 0
End

Function DFF_movies_Button(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	Wave/T DFF_seriesList = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_seriesList
	Wave DFF_selWave = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_selWave
	NVAR firstBackground = root:Globals:imaging:ImageAnalysisGlobals:DFFTab:firstBackground
	NVAR lastBackground=root:Globals:imaging:ImageAnalysisGlobals:DFFTab:lastBackground
	NVAR gRadioVal = root:Globals:imaging:ImageAnalysisGlobals:gRadioVal
	
	switch( ba.eventCode )
		case 2: // mouse up
				DFFMovieList(DFF_seriesList,DFF_selWave,firstBackground,lastBackground)		
			break
	endswitch

	switch(gRadioVal)
		case 1:
			MyCheckProc("All",1)
			break
		case 2:
			MyCheckProc("DFFonly",2)
			break
		case 3: 
			MyCheckProc("NoDFF",3)
			break
					
	EndSwitch	

	return 0
End

Function DFF_QuickTab(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	variable index
	
	switch( ba.eventCode )
		case 2: // mouse up
				String ActiveWindows = WinList("DFFTab*",";","WIN:64")
				
					If(Strlen(ActiveWindows)==0)
						Execute "DFFTab()"
					Else
					
						Do
							String thisWin=StringFromList(index,ActiveWindows)
							
							If(Strlen(thisWin)==0)
								break
							EndIf
							
							KillWindow $thisWin
							
							index+=1
						
						While(1)				
					
					
					EndIf
						
				
			break
	endswitch

	return 0
End

//////////////////////////////////////
// Builds the DFF panel
//////////////////////////////////////
Window DFFTab() : Panel
	PauseUpdate; Silent 1		// building window...
	MakeDFFGlobals()
	NewPanel /W=(505,136,706,468) as "DF/F"
	ModifyPanel cbRGB=(65280,0,0)
	SetDrawLayer UserBack
	DrawText 148,272,"total"
	SetVariable setvar0,pos={12,104},size={81,19},title="Start:",fSize=14
	SetVariable setvar0,format="%g"
	SetVariable setvar0,limits={0,inf,1},value= root:Globals:imaging:ImageAnalysisGlobals:DFFTab:firstBackground
	SetVariable setvar1,pos={107,104},size={81,19},title="End",fSize=14,format="%g"
	SetVariable setvar1,limits={0,inf,1},value= root:Globals:imaging:ImageAnalysisGlobals:DFFTab:lastBackground
	PopupMenu popup0,pos={145,10},size={21,21},proc=DFFPick_popup
	PopupMenu popup0,mode=0,value= #"root:Globals:imaging:ImageAnalysisGlobals:DFFTab:ShowInDFFTab"
	SetVariable setvar2,pos={16,42},size={131,19},proc=DFFStemProc,title="Stem"
	SetVariable setvar2,fSize=13,value= root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_tabStem
	TitleBox DFFWave,pos={18,12},size={40,16},fSize=14,frame=0
	TitleBox DFFWave,variable= root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFFCurrentMovie
	GroupBox group0,pos={8,6},size={186,27},fColor=(65535,65535,65535)
	GroupBox group1,pos={8,37},size={186,27}
	GroupBox group2,pos={8,99},size={186,33}
	Button DFF_oneMovie,pos={11,292},size={88,28},proc=DFF_single_Button,title="\\F'Symbol'D\\F'Arial'F/F single"
	Button DFF_oneMovie,labelBack=(65280,0,0),fColor=(65535,65535,65535)
	Button DFF_oneMovie1,pos={107,292},size={88,28},proc=DFF_movies_Button,title="\\F'Symbol'D\\F'Arial'F/F selection"
	Button DFF_oneMovie1,labelBack=(65280,0,0),fColor=(65535,65535,65535)
	GroupBox group3,pos={8,167},size={186,112}
	ValDisplay valdisp1,pos={109,257},size={32,15},labelBack=(65280,32768,32768)
	ValDisplay valdisp1,fSize=12,frame=0,limits={0,0,0},barmisc={0,1000}
	ValDisplay valdisp1,value= #"root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFFTab_seriesCount"
	ListBox list1,pos={13,176},size={164,69},frame=2
	ListBox list1,listWave=root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_seriesList
	ListBox list1,selWave=root:Globals:imaging:ImageAnalysisGlobals:DFFTab:DFF_selWave,mode= 4
	ListBox list1,editStyle= 2
	Button SelectAllDFF1,pos={16,253},size={68,21},proc=SelectAllDFF_button,title="Select All"
	Button SelectAllDFF1,labelBack=(65280,0,0)
	TitleBox title1,pos={12,148},size={84,16},title="Pick movies",fSize=15,frame=0
	TitleBox title1,fStyle=1
	TitleBox title2,pos={10,79},size={134,16},title="Background frames",fSize=15
	TitleBox title2,frame=0,fStyle=1
EndMacro




//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////


Function ConvertToXY(inwave)
Wave inwave
Duplicate/O inwave X_vals,Y_vals

X_vals[0,*]=DimOffset(inwave,0)+DimDelta(inwave,0)*p

End


//////////////////////// ROI TABS /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////
// Makes Globals for setting ROI
//////////////////////////////////////
Function MakeROIGlobals()

	String dfSav = GetDataFolder(1)

	If(DataFolderExists("root:Globals:imaging:ImageAnalysisGlobals:ROITab"))
		return 0
	Else
		NewDataFolder/O/S root:Globals:imaging:ImageAnalysisGlobals:ROITab
	EndIf
		Variable/G ROI_left
		Variable/G ROI_right
		Variable/G ROI_top
		Variable/G ROI_bottom
		Variable/G V_Marquee
		Variable/G MouseDown
		Variable/G gROI_radioVal 
		String/G ROI_refIMage
		String/G ROI_message 
	SetDataFolder dfSav
End

//////////////////////////////////////
// Updates ROI values based on user-input from a marquee
//////////////////////////////////////
Function UpdateMarqueeValues()
	MakeROIGlobals()
	
	GetMarquee left,bottom
	
	NVAR left = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_left; left=floor(V_left)
	NVAR right = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_right; right = ceil(V_right)
	NVAR top = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_top; top = ceil(V_top)
	NVAR bottom = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_bottom; bottom=floor(V_bottom)
	
	SVAR ROI_refImage = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_refImage
	SVAR message = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_message
	SVAR MovieOnDisplay = root:Globals:imaging:ImageAnalysisGlobals:MovieOnDisplay
		
	ROI_refImage = MovieOnDisplay // So we can append this to a wave note to tell where the focus image came from 
	
	message = "coords: " + "(" + num2str(left) + "," + num2str(bottom) + ") , (" + num2str(right) + "," + num2str(top)+")"				
	Print message
End

//////////////////////////////////////
// Draws a box showing the active ROI
//////////////////////////////////////
Function DrawUsingMarq_coords()

	HideROIDisplay()
	
	DoWindow/F FrameViewer_subWin
	
	SetDrawLayer ProgFront

	SetDrawEnv xcoord= bottom,ycoord= left
	SetDrawEnv linefgc = (65280,65535,65535), fillpat=0,linethick=1.50
	
	
	NVAR left = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_left
	NVAR right = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_right 
	NVAR top = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_top 
	NVAR bottom = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_bottom
	
	If(left==-1)
		return 0
	EndIf
	
	DrawRect left,top,right,bottom
	SetDrawEnv xcoord= bottom,ycoord= left,textrgb=(65280,65535,65535),fsize= 18
	DrawText left,top+1, "ROI" 
	
End

//////////////////////////////////////
// Kills the ROI window w/o asking to Save/Replace
//////////////////////////////////////
Function KillWin_LockROI(infostr)
	String infoStr
	String Event = ""
	Event = StringByKey("EVENT",infostr)
		//Print event
		
	If(stringmatch(Event,"Killvote")) 
		
		MyROICheckProc("LockROI",2)
		DoWindow/K ROITab
	EndIf

End

//////////////////////////////////////
// Hides the shown ROI display (if any available)
//////////////////////////////////////
Function HideROIDisplay()
	SetDrawLayer/K/W=FrameViewer_subWin progFront
End

//////////////////////////////////////
// Kills the active ROI
//////////////////////////////////////
Function KillROI()

	HideROIDisplay()
	
	NVAR left = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_left; left=-1
	NVAR right = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_right; right=-1
	NVAR top = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_top; top=-1
	NVAR bottom = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_bottom;bottom=-1
	
	SVAR ROI_refImage = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_refImage; ROI_refImage=""
	SVAR message = root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_message; message = "no active ROI"
	
End



//////////////////////////////////////
// Hook to indicate user-driven events w/ marquee
//////////////////////////////////////
Function myROIHook(H_Struct)
	STRUCT WMWinHookStruct &H_Struct
	NVAR mouseDown = root:Globals:imaging:ImageAnalysisGlobals:ROITab:mouseDown
	
	If((mouseDown==1) && (StringMatch("Mousemoved",H_Struct.eventName)))
	UpdateMarqueeValues()
	DrawUsingMarq_coords()	
	mouseDown=0
	EndIf
		
	If(Stringmatch("Mousedown",H_Struct.eventName))
		mouseDown=1
	EndIf
	 
End

//////////////////////////////////////
// Sets ROI update mode (locked or able to be updated)
//////////////////////////////////////
Function MyROICheckProc(name,value):CheckBoxControl
	String name
	Variable value
		
	NVAR gROI_radioVal = root:Globals:imaging:ImageAnalysisGlobals:ROITab:gROI_radioVal
	
	Strswitch(name)
		case "EnterROIs":
			gROI_radioVal=1
			SetWindow FrameViewer_subwin,hook(ROI_hk)=myROIHook				
			break
			
		case "LockROI":
			gROI_radioVal=2
			SetWindow FrameViewer_subwin,hook(ROI_hk)=$""
			break
	
	endswitch

	CheckBox EnterROIs,Win=ROITab,value= gROI_radioVal==1
	CheckBox LockROI,Win=ROITab,value= gROI_radioVal==2

End

//////////////////////////////////////
// Kills the active ROI (if any)
//////////////////////////////////////
Function KillROI_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			KillROI()
			break
	endswitch

	return 0
End

//////////////////////////////////////
// Hides the active ROI (if any)
//////////////////////////////////////
Function HideROI_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			HideROIDisplay()
			break
	endswitch

	return 0
End

//////////////////////////////////////
// Shows the active ROI (if any)
//////////////////////////////////////
Function ShowROI_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			DrawUsingMarq_coords()
			break
	endswitch

	return 0
End


//////////////////////////////////////
// 
//////////////////////////////////////
Function ForceLockROI()

MyROICheckProc("LockROI",2)


End

//////////////////////////////////////
// Shows/Hides the ROI tab depending on whether it is currently open
//////////////////////////////////////	
Function ROI_QuickTab(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	variable index
	
	switch( ba.eventCode )
		case 2: // mouse up
				String ActiveWindows = WinList("ROITab*",";","WIN:64")
				
					If(Strlen(ActiveWindows)==0)
						Execute "ROITab()"
					Else
					
						Do
							String thisWin=StringFromList(index,ActiveWindows)
							
							If(Strlen(thisWin)==0)
								break
							EndIf
							
							KillWindow $thisWin
							
							index+=1
						
						While(1)				
					
					
					EndIf
						
				
			break
	endswitch

	return 0
End

//////////////////////////////////////
// Builds the ROI Panel
//////////////////////////////////////
Window ROITab() : Panel
	PauseUpdate; Silent 1		// building window...
	MakeROIGlobals()
	NewPanel /W=(505,136,697,265) as "ROI"
	ModifyPanel cbRGB=(32768,40704,65280)
	SetDrawLayer UserBack
	SetDrawEnv linefgc= (34816,34816,34816),fillpat= 4,fillfgc= (39168,39168,39168),fillbgc= (39168,39168,39168)
	DrawRect 75,10,156,48
	SetDrawEnv linefgc= (34816,34816,34816),fillpat= 0
	DrawRect 18,95,179,118
	DrawText 19,91,"status:"
	CheckBox EnterROIs,pos={79,11},size={75,14},proc=MyROICheckProc,title="Update ROI"
	CheckBox EnterROIs,value= 0,mode=1
	CheckBox LockROI,pos={80,32},size={64,14},proc=MyROICheckProc,title="Lock ROI"
	CheckBox LockROI,value= 1,mode=1
	Button KillROI,pos={17,50},size={45,19},proc=KillROI_ButtonProc,title="Kill"
	Button KillROI1,pos={17,10},size={45,19},proc=ShowROI_ButtonProc,title="Show"
	Button KillROI2,pos={17,30},size={45,19},proc=HideROI_ButtonProc,title="Hide"
	TitleBox title0,pos={29,99},size={119,13},frame=0
	TitleBox title0,variable= root:Globals:imaging:ImageAnalysisGlobals:ROITab:ROI_message
	MyROICheckProc("LockROI",2)
	SetWindow ROITab hook = KillWin_LockROI

EndMacro
