#pragma rtGlobals=1		// Use modern global access method.
#pragma version = 1.0



// Takes wavelist and a sweep number as arguments
// and multiplexes the waves pointed to by the wavelist. Works for 1 and 2d waves.
// Still need to test more.
// 
// Last Modified:21 Aug 2007

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// One of the workhorses of the Acquisition. Takes an input wave list (it will
// almost always be "DAC_combo_list", an input String, sweep number,
// and total points to acquire. The waves of the combo list are multiplexed
// into a 1d wave appropriate for sending out to the ITC18. 
// IMPORTANT:
//  1) the inwave_list argument will almost always be the global DAC_combo_list
//  2) the OutGainString will almost always be DACGainString_out: Only call this
//   fxn after the string has been padded (by InitalizeStringsForSweep()), otherwise
//   you'll multiplex the wrong number of waves
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function Multiplex_wavelist(inwave_list, OutGainString, sweep_num, points_to_acquire)
Wave/T inwave_list
String OutGainString
Variable sweep_num
Variable points_to_acquire

//.....GLOBALS.....//
Wave output_gain_lookup = root:Globals:physiology:DAC_ADC_globals:output_gain_lookup
SVAR DACString_out = root:Globals:physiology:DAC_ADC_globals:DACString_out
//........................//

Variable CurrentGain
Variable j, max_points, is_1d, dims_in_wave, current_sweep

Duplicate/O/T inwave_list wvlist
Variable numDACchannels = strlen(OutGainString)	// .. the number of things to be multiplexed is  = to the DAC sequence length
String ww=""
	
	max_points = points_to_acquire
	Make/O/N=(numDACchannels*max_points) muxed_wave		// Each wave in the list will contribute the same # of points to the muxed wave
															
	For(j=0;j<numDACchannels;j+=1)	// for each DAC channel in the sequence...
		Variable listIsEmpty = dimsize(wvlist,0)>0
		 ww = SelectString (listIsEmpty,"", wvlist[mod(j,dimsize(wvlist,0))])
			
			If(!WaveExists($ww))	// If there's no wave ...
				Make/O/N=(max_points) wv_1d; wv_1d=0	// Make some junk data
			
			Else
				is_1d=dimsize($ww,1)==0 
				
				If(is_1d)
					Duplicate/O $ww wv_1d	// If the wave is 1-d, just grab it....
				
				Else	
					dims_in_wave = dimsize($ww,1)
					current_sweep = mod(sweep_num, dims_in_wave)		// ..otherwise pick out the appropriate 1d wave slice
					Duplicate/O/R=[0,*][current_sweep,current_sweep] $ww wv_1d
				
				EndIf
			
			EndIf 
			
			CurrentGain = (stringmatch(OutGainString[j],"T")) ? 1 : output_gain_lookup[str2num(OutGainString[j])] // look up the gain..
			wv_1d*=CurrentGain
			
			Redimension/N=(max_points) wv_1d
			muxed_wave[j,*;numDACchannels]=wv_1d[(p-j)/numDACchannels]
			
	EndFor

End	


//**************DEMULTIPLEXING**********************//
// takes a 1D muxedWave

Function DeMultiplex(inwave)
Wave inwave

//.....GLOBALS.....//
SVAR ADCstring_out = root:Globals:physiology:DAC_ADC_globals:ADCstring_out
SVAR DACstring_out = root:Globals:physiology:DAC_ADC_globals:DACstring_out
SVAR ADCgainString_out = root:Globals:physiology:DAC_ADC_globals:ADCgainString_out
Wave input_gain_lookup = root:Globals:physiology:DAC_ADC_globals:input_gain_lookup
//........................//

Variable j=0
Variable numChannels = strlen(ADCString_out)
Variable currentGain
//Print "DACOUT:" + " " + DACString_out + " " + "ADCOUT:"+ " " + ADCString_out + " " + "CHANS" + " " + num2str(numChannels)
String currentWaveName = ""
String dfSav = GetDataFolder(1)


MakeADCFolder(0)
//Print "I'm NOW IN FOLDER:" + GetDataFolder(1)
Do
	String ThisChannel=ADCString_out[j]
	
	If(Strlen(ThisChannel)==0)
		break
	Else
		CurrentWaveName="ADC"+ThisChannel
			Make/O/N=(dimsize(inwave,0)/numChannels) $CurrentWaveName
			Wave ThisADCout = $currentWavename; ThisADCout=0
			currentGain = input_gain_lookup[str2num(ADCgainString_out[j])]
			
			/// Set the Units here
			Variable thisGainChoice = str2num(ADCgainString_out[j])
			String DataUnits
			NVAR XScale=root:Globals:physiology:Run_globals:SamplingInterval
			
//			
			Switch (thisGainChoice)
				Case 1:
					DataUnits="nA"
					break
				Case 2:
					DataUnits="mV"
					break
				Default:
					DataUnits=""
			EndSwitch
			
			If(Strlen(DataUnits)>0)
				SetScale d 0, 0,DataUnits, ThisADCOut
			EndIf
				SetScale/P x 0, XScale, "ms", ThisADCOut
				
		
			//Print "Gain For Wave:" + " " + CurrentWaveName +  " " + "IS" + " " + num2str(currentGain)
		
			ThisADCout[0,*]=inwave[j+p*(numChannels)]*currentGain
	EndIf
	j+=1
While(1)
SetDataFolder dfSav
End


Function/S ITCSetSequence([DAC_in_String,ADC_in_String])
String DAC_in_String
String ADC_in_String
String Command=""

//.....GLOBALS.....//	
SVAR DACString_out = root:Globals:physiology:DAC_ADC_globals:DACString_out
SVAR ADCString_out = root:Globals:physiology:DAC_ADC_globals:ADCString_out
//.........................// 

	If(ParamIsDefault(DAC_in_String) && ParamIsDefault(ADC_in_String))
		DAC_in_String = DACString_out
		ADC_in_String = ADCString_out
	EndIf
	
	Command = 	"ITC18Seq\"" +DAC_in_String+ "\","+ "\"" +ADC_in_String+"\""

Return Command
	
End

Function/S ITCSendStim(outwave,inwave,Period)

Wave outwave
Wave inwave
Variable Period
String S_outwave,S_inwave,S_Period
String command

S_outwave = NameOfWave(outwave); S_inwave = NameOfWave(inwave)
S_Period = num2str(Period)

command = "ITC18StimAndSample" + " " + S_outwave + "," + S_inwave + ","+ S_Period + "," + "2," +"0"

Return Command
End


Function MakeADCFolder(switchBack)
Variable switchBack
String savDF = GetDataFolder(1)
	If(DataFolderExists("root:Globals:physiology:ADC_input_waves"))
		SetDataFolder root:Globals:physiology:ADC_input_waves
	Else
		NewDataFolder/S root:Globals:physiology:ADC_input_waves
		Make/O/N=0 ADC0,ADC1,ADC2,ADC3,ADC3,ADC4,ADC5,ADC6,ADC7				
	EndIf
	
	If(switchBack>0)
		SetDataFolder savDF
	EndIf
End
	


//Function Collect_N_sweeps(DeltaT,sweepLength,numSweeps)
//Variable DeltaT,sweepLength,numSweeps
//Variable sweepCounter=0
//Wave muxed_wave = root:Globals:physiology:DAC_ADC_globals:muxed_wave
//Variable points_to_acquire = sweepLength/DeltaT


//SetBackGround Collect_1_Sweep(muxed_wave,DeltaT,sweepLength,sweepCounter)
//sweepCounter+=1

	
End

		
