#pragma rtGlobals=1		// Use modern global access method.
#include "Sweep_Plotter"
#include "DAC_ADC_procs4"
#include "Pulse_Maker"
#include "RunRoutines4"
#include "Multiplex_fxns4"
#include "ImageTabProcs4"
#include "TillProcs"
#include "Camera_Procs"
#include "Frame_Viewer"

// Last Mod: 6/26

/// Main Acquire Menu
//////////////////////////////////////


//////////////////////////////////////
// 
//////////////////////////////////////


Menu "NeuroAcquire"

	"Intialize Physiology", 			AcqPhysInitialize();
	"Initialize Physiology and Imaging", 	AcqPhysAndImageInitialize();
	"____"
	"Main Panel...", 					AcqMakeMainPanel();
	"Switchboard...",					AcqMakeSwitchboard();
	"Grapher ...", 					AcqMakeGrapher();
	"Image Frame Viewer ...",			AcqMakeFrameViewer();
	
End

/// Make necessary folders for experiment Globals
 //////////////////////////////////////
Function InitializeExperiment()

	If(!DataFolderExists("root:Globals"))

		NewDataFolder root:Globals
		NewDataFolder root:Globals:physiology
		NewDataFolder root:Globals:imaging

		MakeRunGlobals()
		MakeImageAnalysisGlobals()
		MakeCameraGlobals()
		MakeCameraParameterGlobals()
		MakeDAC_ADC_Globals()
	EndIf

End

// Open panels necessary for a physiology Experiment
////////////////////////////////////// 
Function AcqPhysInitialize()

	InitializeExperiment()
	AcqMakeMainPanel()
	AcqMakeSwitchboard()
	AcqMakeGrapher()

End


// Open the Image Frame viewer in addition to above panels
// & prompt user for DAC,TTL  channel numbers [camera tigger + monochromator (if applicable)
////////////////////////////////////// 
 
Function AcqPhysAndImageInitialize()


	
	InitializeExperiment()

	NVAR TillLampPresent = root:Globals:physiology:Run_globals:TillLampPresent
	Variable IlluminationSettingsOK = UserIlluminationSettings()
	Variable TriggerSettingsOK = UserTriggerSettings()
	
	If(IlluminationSettingsOK)
		AcqMakeMainPanel()
		AcqMakeSwitchboard()
		AcqMakeGrapher()
		AcqMakeFrameViewer();
	EndIf

	
	InitializeStringsForSweep()
	
	
	If(TillLampPresent==1)
		AcqMakeTillSelector()
	EndIf
	
End

// Prompt User for TTL chan # controlling camera trigger
////////////////////////////////////// 
Function UserTriggerSettings()
	Variable Trigger_UserSelect
	NVAR TriggerChannel=root:Globals:physiology:Run_globals:TriggerChannel

	Prompt Trigger_UserSelect, "Which TTL channel triggers the camera?", popup, "0;1;2;3"
	DoPrompt "Camera Triggering", Trigger_UserSelect

	If(V_flag==0)
		TriggerChannel = Trigger_UserSelect-1
	Else
		return 0
	EndIf

	return 1

End

// Prompt User for DAC [if using monochromator] or TTL [if using Xe/Hg lamp] Chan #
// controlling light source for imaging
//////////////////////////////////////  
Function UserIlluminationSettings()
	Variable UsingTillLamp, DAC_chan,TTL_chan
	NVAR TillLampPresent = root:Globals:physiology:Run_globals:TillLampPresent
	NVAR IlluminationChannel = root:Globals:physiology:Run_globals:IlluminationChannel
	

	Prompt UsingTillLamp, " Using Monochromator?", popup, "Yes;No"
	Prompt DAC_chan, "Which DAC channel controls the monochromator? ", popup, "0;1;2;3"
	Prompt TTL_chan, "Which TTL_channel controls the shutter?", popup, "0;1;2;3"
 
	DoPrompt "Camera Settings", UsingTillLamp

	If(V_flag==0)
		If(UsingTillLamp==1)
			TillLampPresent=1
			//AcqMakeTillSelector()
			
			DoPrompt "Active DAC", DAC_chan
		
			If(V_flag==0)
				IlluminationChannel=DAC_chan-1
			Else
				return 0
			EndIf
		
		Else
			TillLampPresent=0
			DoPrompt "Active TTL", TTL_chan
			If(V_flag==0)
				IlluminationChannel=TTL_chan-1
			Else
				return 0
			EndIf
		
		EndIf
	Else
		return 0
	EndIf

	return 1

End
 