#pragma rtGlobals=1		// Use modern global access method.
#pragma version 1.0

// Last modified: 6/26/08 (JBC)


//////////////////////////////////////
// Set up Pulse_Maker globals
//////////////////////////////////////  
Function MakeStim_MakerGlobals()

	String dfSav = GetDataFolder(1)
      
	If(!dataFolderExists("root:Packages"))
		NewDataFolder root:Packages
	EndIf
        
	If(!dataFolderExists("root:Packages:Stim_Maker_globals"))
        	
		NewDataFolder/S root:Packages:Stim_Maker_globals
              
		String/G SM_pulse_name
		String/G SM_pulseSeries_mode
              
		Variable/G SM_delay_before_pulse
		Variable/G SM_number_of_pulses
		Variable/G SM_duration_of_pulses
		Variable/G SM_interval_between_pulses
		Variable/G SM_time_after_pulses
		Variable/G SM_pulse_amplitude
		Variable/G SM_start
		Variable/G SM_step
		Variable/G SM_number
        
	Endif 
        
	SetDataFolder dfSav
End

//////////////////////////////////////
// Make a single pulse with specified timing/amplitude 
//////////////////////////////////////              
Function MakePulse(del,num,dur,int,time_post,amp,sampRate,name)	
	// Pulse attributes: 
	Variable del	// delay (ms)
	Variable num // number of pulses
	Variable dur  // duration of pulses (ms)
	Variable int // interval between pulses (ms)
	Variable time_post // time after pulses (ms)
	Variable amp // amplitude (depends on recording mode: for Iclamp will be pA, Vclamp will be mV, etc..)
	String name	// name of pulse to make
	Variable sampRate // sampling Rate (ms)
   	
	Variable n=0
        
	Variable samplingScale = 1/SampRate
	Variable total_time = del+num*(dur+int)+time_post
	Variable total_points = total_time*samplingScale
        
	String dfSav =GetDataFolder(1)
	SetDataFolder root:	// always make pulses in root: so user can easily manipulate during experiment w/o switching data folders
       
	Make/O/N=(total_points) $name
	SetScale/P x, 0, sampRate, $name	
        
	Wave w1 = $name;  w1 = 0
        
	Do
		w1[x2pnt(w1,del+(n*(int+dur))),x2pnt(w1,del+(n*(int+dur))+dur)] = amp
		n=n+1
	While (n<num)
        
	SetDataFolder dfSav
	
End

//////////////////////////////////////
//Make a pulse series (2D wave) w/ specified attributes
//////////////////////////////////////     
Function MakeSeries(del,num,dur,int,time_post,amp,sampRate,start,step,number,mode,name)
	// Pulse attributes:
	Variable del	// delay (ms)
	Variable num // number of pulses
	Variable dur  // duration of pulses (ms)
	Variable int // interval between pulses (ms)
	Variable time_post // time after pulses (ms)
	Variable amp // amplitude (depends on recording mode: for Iclamp will be pA, Vclamp will be mV, etc..)
	String name	// name of pulse to make
	Variable sampRate // sampling Rate (ms)

	// Series Variables:
	Variable start	 
	Variable step  
	Variable number // number of sweeps in 2d-series
	String mode // series variable (amplitude, delay, num pulses, etc..)
 
	Variable n=0

	String dfSav = GetDataFolder(1)
	SetDataFolder root:
	
	If(WaveExists($name))
		Redimension/N=(1,0) $name
	EndIf
	
	Strswitch (mode)
        
		case "amplitude":        
			//Killwaves/Z $name
			Do 
				MakePulse(del,num,dur,int,time_post,start+(step*n),sampRate,"SM_temp")
				PackageTo2d($"SM_temp",name)
				n+=1
			While (n<number)
			break
                  
		case "delay":
			//Killwaves/Z $name
			Do
				MakePulse(start+(step*n),num,dur,int,time_post,amp,sampRate,"SM_temp")
				PackageTo2d($"SM_temp",name)
				n+=1
			While (n<number)
			break
          
		case "pulse number":
			//Killwaves/Z $name
			Do
				MakePulse(del,start+(step*n),dur,int,time_post,amp,sampRate,"SM_temp")
				PackageTo2d($"SM_temp",name)
				n+=1
			While (n<number)
			break
          
		case "duration":
			//Killwaves/Z $name
			Do
				MakePulse(del,num,start+(step*n),int,time_post,amp,sampRate,"SM_temp")
				PackageTo2d($"SM_temp",name)
				n+=1
			While (n<number)
			break
          
		case "interval":
			//Killwaves/Z $name
			Do 
				MakePulse(del,num,dur,start+(step*n),time_post,amp,sampRate,"SM_temp")
				PackageTo2d($"SM_temp",name)
				n+=1
			While (n<number)
			break
            
		default:
			return 0
            			
       endswitch
	
	SetScale/P x,0,0.1,$name
	SetDataFolder dfSav
	
End

//////////////////////////////////////
// appends a 1-d wav (apWave) onto an existing 2D wave (outwave)
// & redimensions as appropriate.
//////////////////////////////////////    
Function PackageTo2D(apWave,outWave)
	Wave apWave
	String outWave

	Duplicate/O apWave appendWave

	Variable pts_outwave_X=dimsize($outwave,0)
	Variable pts_outwave_Y=dimsize($outwave,1)
	Variable pts_apwave = dimsize(appendWave,0)

	If (WaveExists($outWave)==0)
		Make/O/N=(dimsize(appendWave,0),0) $outWave
		Redimension/N=(dimsize($outWave,0),1) $outWave
		Wave tmp=$outwave
		tmp=appendWave
		return 0
	Endif

	If ((WaveExists($outWave)) && (pts_apwave>=pts_outwave_X))
               
		Redimension/N =(pts_apwave,pts_outwave_Y+1) $outWave
		Wave w1=$outwave
		w1[0,*][pts_outwave_Y,pts_outwave_Y]=0
                
	Elseif ((WaveExists($outWave)) && (pts_apwave<pts_outwave_X))
        
		Redimension/N=(pts_outwave_X) appendWave
		Redimension/N= (pts_outwave_X, pts_outwave_Y+1) $outWave
		Wave w1=$outWave
		w1[0,*][pts_outwave_Y,pts_outwave_Y]=0
	Endif

	w1[0,*][dimsize(w1,1)-1,dimsize(w1,1)-1]=appendWave[p]

End



//////////////////////////////////////////// PANELS AND BUTTONS //////////////////////////////////


//////////////////////////////////////
// Make the Stim Maker Panel
//////////////////////////////////////  
Function AcqMakeStimMaker()

	MakeStim_MakerGlobals()
	
	DoWindow/F StimMaker
	If(V_Flag==1)
		return 0
	EndIf
	
	PauseUpdate; Silent 1		// building window...
	NewPanel/N=StimMaker /W=(1277,201,1565,476) as "Stimulus Maker"
	SetVariable setvar0,pos={26,25},size={202,20},title="Pulse Name",fSize=14
	SetVariable setvar0,value= root:Packages:Stim_Maker_globals:SM_pulse_name
	GroupBox group0,pos={7,50},size={272,104},labelBack=(52224,52224,52224)
	GroupBox group0,fColor=(65535,65535,65535)
	SetVariable Delay_before_pulses1,pos={20,61},size={140,16},title="Pulse delay"
	SetVariable Delay_before_pulses1,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_delay_before_pulse
	SetVariable number_of_pulses1,pos={169,82},size={99,16},title="# pulses"
	SetVariable number_of_pulses1,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_number_of_pulses
	SetVariable duration_of_pulses1,pos={169,60},size={99,16},title="Duration "
	SetVariable duration_of_pulses1,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_duration_of_pulses
	SetVariable Interval_between_pulses1,pos={20,107},size={140,16},title="Inter-pulse interval"
	SetVariable Interval_between_pulses1,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_interval_between_pulses
	SetVariable Time_after_pulse1,pos={19,83},size={140,16},title="Time after pulses"
	SetVariable Time_after_pulse1,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_time_after_pulses
	SetVariable pulse_amplitude1,pos={20,130},size={140,16},title="Amplitude"
	SetVariable pulse_amplitude1,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_pulse_amplitude
	TitleBox Series,pos={10,165},size={39,16},title="Series",fSize=14,frame=0
	TitleBox Series,fStyle=0
	PopupMenu templates,pos={233,23},size={21,21},proc=P_SM_pulseTemplates,fSize=16
	PopupMenu templates,mode=0,value= #"\"IV_2d_outwave;num_2d_outwave;siu_pulse;trig_pulse\""
	Button tab0_make_pulse,pos={172,110},size={94,37},proc=SM_MakePulse,title="Make pulse"
	Button tab0_make_pulse,fColor=(65535,65535,65535)
	GroupBox group1,pos={6,185},size={272,83},labelBack=(52224,52224,52224)
	GroupBox group1,fColor=(65535,65535,65535)
	SetVariable Start3,pos={17,193},size={70,16},title="Start"
	SetVariable Start3,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_start
	SetVariable Step3,pos={111,193},size={67,16},title="Step"
	SetVariable Step3,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_step
	SetVariable Number3,pos={201,193},size={70,16},title="Number"
	SetVariable Number3,limits={-inf,inf,0},value= root:Packages:Stim_Maker_globals:SM_number
	PopupMenu PV,pos={16,229},size={143,21},proc=SM_pulseSeriesVariable,title="Variable"
	PopupMenu PV,mode=3,popvalue="pulse number",value= #"\"amplitude;delay;pulse number;duration;interval\""
	Button tab0_make_pulse_serie1,pos={175,223},size={94,37},proc=SM_makePulseSeriesButton,title="Make Pulse Series"
	Button tab0_make_pulse_serie1,fColor=(65535,65535,65535)
	
End

//////////////////////////////////////
// Sets the series Mode (i.e. is the series variable amp, dur, num, etc..
//////////////////////////////////////    
Function SM_pulseSeriesVariable(ctrlName,popNum,popStr) : PopupMenuControl
        String ctrlName
        Variable popNum
        String popStr
        
       SVAR SM_pulseSeries_mode = root:Packages:Stim_Maker_globals:SM_pulseSeries_mode
    	 SM_pulseSeries_mode = popStr
      
End

//////////////////////////////////////
// Make a pulse series according to global values
//////////////////////////////////////    
Function SM_makePulseSeriesButton(ctrlName) : ButtonControl
	String ctrlName
       MakeStim_MakerGlobals()
	
	// Globals //  
	NVAR del= root:Packages:Stim_Maker_globals:SM_delay_before_pulse
	NVAR num= root:Packages:Stim_Maker_globals:SM_number_of_pulses
	NVAR dur= root:Packages:Stim_Maker_globals:SM_duration_of_pulses
	NVAR int= root:Packages:Stim_Maker_globals:SM_interval_between_pulses
	NVAR time_post= root:Packages:Stim_Maker_globals:SM_time_after_pulses
	NVAR amp = root:Packages:Stim_Maker_globals:SM_pulse_amplitude
	NVAR start= root:Packages:Stim_Maker_globals:SM_start
	NVAR step= root:Packages:Stim_Maker_globals:SM_step
	NVAR number= root:Packages:Stim_Maker_globals:SM_number
	
	SVAR name=root:Packages:Stim_Maker_globals:SM_pulse_name
	SVAR mode = root:Packages:Stim_Maker_globals:SM_pulseSeries_mode
	// 		 //  
	
	Variable sampRate=NumVarOrDefault ("root:Globals:physiology:Run_globals:SamplingInterval",.1)
     
       MakeSeries(del,num,dur,int,time_post,amp,sampRate,start,step,number,mode,name)
     
     	If(dimsize($name,1)>1)
     	
     	NewWaterfall $name
     	EndIf
     	
End

//////////////////////////////////////
// Make a single pulse according to global values
//////////////////////////////////////    
Function SM_MakePulse(ctrlName) : ButtonControl
	String ctrlName

	NVAR del= root:Packages:Stim_Maker_globals:SM_delay_before_pulse
	NVAR num= root:Packages:Stim_Maker_globals:SM_number_of_pulses
	NVAR dur= root:Packages:Stim_Maker_globals:SM_duration_of_pulses
	NVAR int= root:Packages:Stim_Maker_globals:SM_interval_between_pulses
	NVAR time_post= root:Packages:Stim_Maker_globals:SM_time_after_pulses
	NVAR amp = root:Packages:Stim_Maker_globals:SM_pulse_amplitude
	
	Variable sampRate=NumVarOrDefault ("root:Globals:physiology:Run_globals:SamplingInterval",.1)
	SVAR name=root:Packages:Stim_Maker_globals:SM_pulse_name
	
	MakePulse(del,num,dur,int,time_post,amp,sampRate,name)
	Display $name
	//
End




//////////////////////////////////////
// Set Global PM variables to some commonly used values 
// for specific types of stim protocols
//////////////////////////////////////  
Function P_SM_pulseTemplates(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	MakeStim_MakerGlobals()
	// Globals //  
	NVAR del= root:Packages:Stim_Maker_globals:SM_delay_before_pulse
	NVAR num= root:Packages:Stim_Maker_globals:SM_number_of_pulses
	NVAR dur= root:Packages:Stim_Maker_globals:SM_duration_of_pulses
	NVAR int= root:Packages:Stim_Maker_globals:SM_interval_between_pulses
	NVAR time_post= root:Packages:Stim_Maker_globals:SM_time_after_pulses
	NVAR amp = root:Packages:Stim_Maker_globals:SM_pulse_amplitude
	NVAR start= root:Packages:Stim_Maker_globals:SM_start
	NVAR step= root:Packages:Stim_Maker_globals:SM_step
	NVAR number= root:Packages:Stim_Maker_globals:SM_number
	
	SVAR name=root:Packages:Stim_Maker_globals:SM_pulse_name
	SVAR mode = root:Packages:Stim_Maker_globals:SM_pulseSeries_mode
	// 		 //  

	StrSwitch(popStr)
      
		Case "IV_2d_outwave":  
			name = "IV_2d_outwave"
			del = 50
			num =1
			dur = 100
			int = 0
			time_post = 50
			start = -50
			step = 50 
			number = 6
			amp = 100
		
			SM_pulseSeriesVariable("PulseVariable",1,"amplitude")
		
			ControlUpdate PV 
			PopUpMenu PV mode=1
        	
			break
      
		Case "num_2d_outwave": 
			name = "num_2d_outwave"
			del = 100
			num =1
			dur = 3
			int = 40
			time_post = 10
			start = 1
			step = 2 
			number = 6
			amp = 1000
		
			SM_pulseSeriesVariable("PulseVariable",3,"pulse number")

			ControlUpdate PV 
			PopupMenu PV mode=3
			
			break
	  
		Case "siu_pulse": 
			name = "siu_pulse"
			del = 500
			num =1
			dur = .1
			int = 40
			time_post = 10
			amp=1
		     
			break
	
		Case "trig_pulse": 
			name = "trig_pulse"
			del = 1
			num =1
			dur = .1
			int = 40
			time_post = 10
			amp=1
      
			break
		
		default:
			
			break
	
	endswitch
	
End
