#pragma rtGlobals=1		// Use modern global access method
#pragma version=1.0

// These functions allow you to plot 1-D waves from a chosen data folder to
// tile or stack traces in several configurations.
 
// Last Modified: (JBC)


//////////////////////////////////////
/// Makes necessary globals for plotting multiple tiled sweeps
//////////////////////////////////////
Function MakePlotterGlobals()
	String dfSav = GetDataFolder(1)

	If(!DataFolderExists("root:Globals:physiology:GrapherGlobals"))
		
		NewDataFolder/S root:Globals:physiology:GrapherGlobals
		Make/O/T ADCAvailableList="",WaveListToPlot="",GraphAppendTable="",WavesToGraph="" 
		
		Make/N=(8,3) MyColors 
		
		MyColors[][0]={65280,0,0,0,26112,26368,0,65280}
		MyColors[][1]={0,0,65280,0,26112,0,52224,65280}
		MyColors[][2]={0,0,0,65280,26112,52224,26368,0}
		
		Make/O selWave
		
		Variable/G BottomIsOn
		Variable/G numGraphs
		Variable/G graphsToShow
		
	EndIf

	SetDataFolder dfSav
End


//////////////////////////////////////
///Redimensions text waves needed for proper display of sweeps
//////////////////////////////////////
Function ResizePlotterChoices(numGraphs,ADCAvailableList,WavesToGraph,selWave)
	Variable numGraphs
	Wave/T ADCAvailableList
	Wave/T WavesToGraph 
	Wave selWave

	Variable RowsInList=dimsize(ADCAvailableList,0)
	Variable n
	String s

	If (numGraphs>4 || numGraphs<1) 
		return 0
	EndIf

	// Adjust Waves for Listbox Display	
	Redimension/N=(RowsInList+1,numGraphs) WavesToGraph	
	Redimension/N=(RowsInList+1,numGraphs,2) selwave
	WavesToGraph[0,*][0]=ADCAvailableList[p]
	WavesToGraph[dimsize(WavesToGraph,0)-1][0]="Right Append?" 

	If(numGraphs>1)
		WavesToGraph[0,*][1,*]=WavesToGraph[p][0]
	EndIf

	For(n=0;n<dimsize(WavesToGraph,1);n+=1)
		
		sprintf s,"Graph"+num2str(n+1)		// Necessary for proper Listbox display 
		SetDimLabel 1,n,$s,WavesToGraph
	
	EndFor

	selWave[0,*][0,*][0,*]=0	
	selWave[0,*][0,*][0] = 100
	selWave[dimsize(ADCAvailableList,0)][0,*][1]=1 // Set bit on third dim to specify display properties

End

//////////////////////////////////////
/// Updates the Append Table (GAT below) to refelct user display choices
//////////////////////////////////////
Function UpdatePlotterDisplaySettings(numGraphs,WavesToGraph,GAT,selwave)

	Variable numGraphs
	Wave/T WavesToGraph 
	Wave/T GAT 
	Wave selwave 
	
	Variable spacing = .8/numGraphs 	// 80% of total graph height is devoted to axes
	Variable AxisSeparation = .2/(numGraphs-1)	// .. and 20% is saved for white space

	Variable RightAppendIndex=1
	Variable LeftAppendIndex=1
	Variable n,j
	Variable FirstTraceOnGraph

	Variable TraceIsFirstInColumn
	Variable TraceIsFirstOnGraph=0

	
	Redimension/N=(dimsize(WavesToGraph,0-1),dimsize(WavesToGraph,1)) GAT // Make sure the append table 
	// (see above) is same size as wave choices
	GAT=""

	If(numGraphs==1)	// Only one graph? Use the whole area for display
		spacing=1
		AxisSeparation=0
	EndIf
	
	For(n=0;n<dimsize(GAT,1);n+=1)	// Loop through the columns
		
		String IsCheckedString = DecimalToBinary(selwave[dimsize(selwave,0)][n][0])	
		String RightAppendBit = IsCheckedString[strlen(IsCheckedString)-5]
		Variable AppendOnRight = StringMatch(RightAppendBit,"1")
		
		TraceIsFirstInColumn=0
		
		For(j=0;j<dimsize(GAT,0)-1;j+=1) // Loop through each row in a column
			
			String ListCheck =DecimalToBinary(selwave[j][n][0])
			String ListCheckBit = ListCheck[strlen(ListCheck)-5]
			Variable ItemIsChecked = StringMatch(ListCheckBit,"1")		
					
			If (ItemIsChecked)
				
				If((n==0) & (AppendOnRight))
					
					If(TraceIsFirstInColumn==0)
						
						GAT[j][n]="left"
						TraceIsFirstInColumn+=1
					
					ElseIf(TraceIsFirstInColumn!=0)
						
						GAT[j][n]="right"
					
					EndIf
				
				ElseIf((n==0) & !(AppendOnRight))
					
					GAT[j][n]="left"
				
				EndIf
					
				If((n!=0) & (AppendOnRight))
					
					If(TraceIsFirstInColumn==0)
					
						GAT[j][n]="L"+num2str(n)
					
						TraceIsFirstInColumn+=1
					
					ElseIf(TraceIsFirstInColumn!=0)
					
						GAT[j][n]="R"+num2str(n)
					
					EndIf
				
				ElseIf((n!=0) & !(AppendOnRight))
				
					GAT[j][n]="L"+num2str(n)
				
				EndIf	
		
			EndIf				
		
		EndFor
	
	EndFor			
				
End



//////////////////////////////////////
/// Appends a trace to a graph. This is basically just a big look up table 
/// that makes some code above cleaner.
//////////////////////////////////////
Function AppendTracesUsingList(inwave,AppendMode,bottomsOn)
	Wave inwave
	String AppendMode
	Variable bottomsOn

	strswitch (AppendMode)

		case "left":
			If(bottomsOn)
				AppendToGraph/L=left/B=bottom inwave
				break
			Else
				AppendToGraph/L=left inwave
				break
			EndIf
		case "L1":
			If(bottomsOn)
				AppendToGraph/L=L1/B=B1 inwave
				break
			Else
				AppendToGraph/L=L1 inwave
				break
			Endif
		case "L2":
			If(bottomsOn)
				AppendToGraph/L=L2/B=B2 inwave
				break
			Else
				AppendToGraph/L=L2 inwave
				break
			EndIf
		case "L3":
			If(bottomsOn)
				AppendToGraph/L=L3/B=B3 inwave
				break
			Else
				AppendToGraph/L=L3 inwave
				break
			EndIf
		case "right":
			If(bottomsOn)
				AppendToGraph/R=right/B=bottom inwave
				break
			Else
				AppendToGraph/R=right inwave
				break
			EndIf
		case "R1":
			If(bottomsOn)
				AppendToGraph/R=R1/B=B1 inwave
				break
			Else
				AppendToGraph/R=R1 inwave
				break
			EndIf
		case "R2":
			If(bottomsOn)
				AppendToGraph/R=R2/B=B2 inwave
				break
			Else
				AppendToGraph/R=R2 inwave
				break
			EndIf
		case "R3":
			If(bottomsOn)
				AppendToGraph/R=R3/B=B3 inwave
				break
			Else
				AppendToGraph/R=R3 inwave
				break
			EndIf
		default:
			break
	EndSwitch

End


//////////////////////////////////////
/// Plots the waves listed as entries of WaveListToPlot according to the
/// parameters specified in GraphAppendTable.
//////////////////////////////////////
Function UpdateTracesOnPlotter(WaveListToPlot,GraphAppendTable,BottomIsOn,destFolder)
	Wave/T WaveListToPlot
	Wave/T GraphAppendTable
	Variable BottomIsOn
	String destFolder

	Variable spacing = .8/(dimsize(WaveListToPlot,1))
	Variable AxisSeparation = .2/(dimsize(WaveListToPlot,1)-1)
	Variable lowAxis,hiAxis

	Variable n,j

	String DisplayedGraphs = WinList("*",";","WIN:1")

	If(StringMatch(DisplayedGraphs,"*Sweeps*"))	// Make sure things only done to the "Sweeps" windows
		DoWindow/F Sweeps
	Else
		Display/N=Sweeps as "Sweeps"
	EndIf

	ClearTracesOnGraph()

	If(dimsize(WaveListToPlot,1)==1)	// Not using any subgraphs? 
		spacing = 1					// .. use all the whitespace
		AxisSeparation = 0
	EndIf
	
	For(n=0;n<dimsize(GraphAppendTable,1);n+=1)	// For each column (distinct subgraph).....
		
		lowAxis=(n)*(AxisSeparation+spacing) // .. all traces will be on the same axis range, defined here
		hiAxis= lowAxis + spacing
		
		For(j=0;j<(dimsize(GraphAppendTable,0));j+=1)	// For each row (all traces in the same subgraph)
			
			String ThisAxisRef = GraphAppendTable[j][n]    // Graph on appropriate axes (e.g. left, L1,R2...)
			
			If(Strlen(ThisAxisRef)!=0)				
				String ThisWaveRef = destFolder+WaveListToPlot[j][n]
				Wave ww = $ThisWaveRef
					If(!WaveExists(ww))
						return 0
					EndIf
				AppendTracesUsingList(ww,ThisAxisRef,bottomIsOn)
				ModifyGraph standoff($thisAxisRef)=1,axisEnab($thisAxisRef)={lowAxis,hiAxis},FreePos($thisAxisRef)=0,lblPosMode($thisAxisRef)=1
		
				If(BottomIsOn) // Does user want to show time-axis for all traces, or just the bottom one?
					String BottomAxisRef =BottomAxisFromLeft(ThisAxisRef)
					ModifyGraph 	freePos($BottomAxisRef)={lowAxis,kwFraction}	
				EndIf			
			EndIf
		
		EndFor
	
	EndFor	
	
End

//////////////////////////////////////
/// Does what it says.
//////////////////////////////////////	
Function ClearTracesOnGraph()
	Variable i=0
	Variable n=0

	Do
		String TracesOnGraph = TraceNameList("",";",1)	
		Do
			String CurrentTrace = StringFromList(i,TracesOnGraph)
			If(Strlen(CurrentTrace)==0)
				break
			Else
				RemoveFromGraph/Z $CurrentTrace
			EndIf
			i+=1
		
		While(1)
		i=0
	While(Strlen(TracesOnGraph)>0)
End		

//////////////////////////////////////
/// Returns a string that supplies a bottom axis 
/// reference for use in fxns above
//////////////////////////////////////			
Function/S BottomAxisFromLeft(inString)
	String inString
	String outString=""


	StrSwitch(inString)
		case "left":
			outString = "bottom"
			return outString
		
		case "right":
			outString = "bottom"
			return outString
	
		default:
			break
	
	EndSwitch
	
	If(StringMatch(inString,"R*"))		
		outString = ReplaceString("R",inString,"B")
	ElseIf (Stringmatch(inString,"L*"))
		outString = ReplaceString("L",inString,"B")
	EndIf
	
	return outString
End
	
//////////////////////////////////////				
// Perhaps belongs in a different proc.... 
// Also this is very Kludgy.....
//////////////////////////////////////			
Function SetADCAvailableList()

	//////------Globals-----------///////
	SVAR ADC_String_preBuffer = root:Globals:physiology:DAC_ADC_globals:ADC_string_preBuffer
	/////////////////////////////////////////

	Variable index

	Make/O/T/N=(strlen(ADC_String_preBuffer),0) root:Globals:physiology:GrapherGlobals:ADCAvailableList
	wave/T ADCs_available = root:Globals:physiology:GrapherGlobals:ADCAvailableList

	Do
		ADCs_available[index]="ADC"+ADC_String_preBuffer[index]
		index+=1

	While(index<strlen(ADC_String_preBuffer))
	
End
	

//////////////////////////////////////	
// Colorizes traces based on 
// trace name
////////////////////////////////////		
Function UpdateGraphColors(myColors)
	Wave myColors
	String TracesOnGraph = TraceNameList("",";",1)

	//Wave myColors = root:Globals:physiology:GrapherGlobals:myColors
	Variable index

	Do
		String thisTrace = StringFromList(index,TracesOnGraph)
		If(strlen(thisTrace)==0)
			break
		EndIf
		
		String ThisWave = NameOfWave(TraceNameToWaveRef("",thisTrace))
		Variable ColorIndex = str2num(ThisWave[strlen(ThisWave)-1])
		
		If(ColorIndex!=NaN)
		
			ModifyGraph rgb ($ThisTrace)=(MyColors[ColorIndex][0],MyColors[ColorIndex][1],MyColors[ColorIndex][2])
		
		EndIf
		
		index+=1
	While(1)
	
End


//////////////////////////////////////	
// Adds/Updates legend on 
// Graph of ADC sweeps
////////////////////////////////////	
Function SmartPlotterLegend(myColors)
	Wave myColors
	String TracesOnGraph = TraceNameList("",";",1)
	String TextBoxString=""
	
	Variable index
	
	
		
	Do
		String thisTrace=StringFromList(index,TracesOnGraph)
		
		If(strlen(thisTrace)==0)
			break
		EndIf
		
		If(!stringmatch(thisTrace,"*#*"))
			String thisWaveName = StringFromList(index,TracesOnGraph)
			Variable thisColor = str2num(thisWaveName[strlen(thisWaveName)-1])
			String ColorString = "\\K" +"("+num2str(myColors[thisColor][0])+","+num2str(myColors[thisColor][1])+","+num2str(myColors[thisColor][2])+")"
			
			TextBoxString=TextBoxString+ColorString+"ADC "+thisWaveName[strlen(thisWaveName)-1]+ "     "
				
			TextBox/C/N=fakeLegend/F=0/A=MT/E TextBoxString  
		EndIf
		index+=1
	While(1)
	
End
	

					// PANELS AND CONTROLS //
///////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

Function WaveChooser_ListBox(ctrlName,row,col,event) : ListBoxControl
	String ctrlName
	Variable row
	Variable col
	Variable event	//1=mouse down, 2=up, 3=dbl click, 4=cell select with mouse or keys
	//5=cell select with shift key, 6=begin edit, 7=end
					
	If(Strlen(WinList("*Sweeps*",";",""))>0 && event<3)
		TextBox/W=Sweeps/K/N=fakeLegend				
	EndIf
	
	If (event<3)
		
		Wave/T WavesToGraph = root:Globals:physiology:GrapherGlobals:WavesToGraph
		Wave/T GraphAppendTable = root:Globals:physiology:GrapherGlobals:GraphAppendTable
		Wave selWave = root:Globals:physiology:GrapherGlobals:selWave
		NVAR bottomIsOn = root:Globals:physiology:GrapherGlobals:bottomIsOn
		NVAR numGraphs = root:Globals:physiology:GrapherGlobals:numGraphs
		Wave/T myColors = root:Globals:physiology:GrapherGlobals:myColors
		
		String destFolder = "root:Globals:physiology:ADC_input_waves:"
			
		If(!dimsize(ADCAvailableList,0)==0)
			
			UpdatePlotterDisplaySettings(numGraphs,WavesToGraph,GraphAppendTable,selWave)
			UpdateTracesOnPlotter(WavesToGraph,GraphAppendTable,bottomIsOn,destFolder)
			UpdateGraphColors(myColors)
			SmartPlotterLegend(myColors)
			
		EndIf
	EndIf

	return 0

End


///////////////////////////// The grapher panel ////////////////////////////////////

Function AcqMakeGrapher() : Panel
	
	InitializeExperiment()
	
	DoWindow/F Sweep_Plotter
	If(V_Flag==1)
		return 0
	EndIf
		

	MakePlotterGlobals()
		
	Wave/T WavesToGraph = root:Globals:physiology:GrapherGlobals:WavesToGraph
	Wave/T GraphAppendTable = root:Globals:physiology:GrapherGlobals:GraphAppendTable
	Wave selWave = root:Globals:physiology:GrapherGlobals:selWave
	NVAR bottomIsOn = root:Globals:physiology:GrapherGlobals:bottomIsOn
	NVAR numGraphs = root:Globals:physiology:GrapherGlobals:numGraphs	
		
	UpdatePlotterDisplaySettings(numGraphs,WavesToGraph,GraphAppendTable,selWave)
	SetADCAvailableList()
	ChooseGraphNumber("Number_of_graphs",1,"1")
	
	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(930,76,1195,274)/K=1/N=Sweep_Plotter as "Sweep Plotter"
	PopupMenu Number_of_graphs,pos={22,11},size={134,21},proc=ChooseGraphNumber,title="Number of Graphs"
	PopupMenu Number_of_graphs,mode=2,popvalue="2",value= #"\"1;2;3;4\""
	ListBox GraphChoices,pos={23,42},size={217,118},proc=WaveChooser_ListBox
	ListBox GraphChoices,listWave=root:Globals:physiology:GrapherGlobals:WavesToGraph
	ListBox GraphChoices,selWave=root:Globals:physiology:GrapherGlobals:SelWave,mode= 8,widths={80}
	CheckBox bottomAxes_check,pos={26,172},size={141,14},title="bottom axes for all traces?"
	CheckBox bottomAxes_check,variable= root:Globals:physiology:GrapherGlobals:BottomIsOn
	
End



Function ChooseGraphNumber(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	Wave/T ADCAvailableList = root:globals:Physiology:GrapherGlobals:ADCAvailableList
	Wave/T WavesToGraph = root:globals:Physiology:GrapherGlobals:WavesToGraph
	Wave selWave = root:globals:Physiology:GrapherGlobals:selWave
	
	ResizePlotterChoices(popNum, ADCAvailableList,WavesToGraph,selWave)
	
	NVAR numGraphs = root:Globals:physiology:GrapherGlobals:numGraphs
	
	numGraphs = popNum
End

//////////////Makes some calculations easier... probably a bit out of place here /////////////////

Function/S DecimalToBinary(decNumber)
	Variable decNumber
	Variable Root=decNumber
	String binaryString =""

	Do
		
		If(mod(root,2)!=0)
			binaryString = AddListItem ("1",binaryString,";")
			Root-=1
		Else
			binaryString = AddListItem ("0", binaryString,";")
		EndIf
	
		Root=Floor(Root/2)

	While(Root!=0)
	binaryString=ReplaceString (";",binaryString,"")
	Return binaryString

End