#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.0

// Last Modified 9/26/07 (JBC)


//////////////////////////////////////
// Make Globals for Manual adjustment of monochromator
//////////////////////////////////////
Function MakeTillGlobals()

	String dfSav=GetDataFolder(1)
	
	If(!DataFolderExists("root:Packages"))
		NewDataFolder root:Packages
	EndIf
	
	If(!DataFolderExists("root:Packages:TillProcGlobals"))
	
		NewDataFolder/O/S root:Packages:TillProcGlobals
		
		Variable/G TillWavelengthChan
		Make/O/N=7 TillLamp_TickVals;TillLamp_TickVals={320,420,480,540,600,660,720}
		Make/O/T/N=7 TillLamp_TickLabels
			TillLamp_TickLabels[0] ="320" 
			TillLamp_TickLabels[1] ="420"
			TillLamp_TickLabels[2] = "\f01\K(0,0,65280)480"
			TillLamp_TickLabels[3] = "\f01\K(0,65280,0)540"
			TillLamp_TickLabels[4] = "\f01\K(65280,52224,0)600"
			TillLamp_TickLabels[5] = "\f01\K(65280,21760,0)660"
			TillLamp_TickLabels[6] = "\f01\K(65280,0,0)720"
				
	EndIf
		
	SetDataFolder dfSav		
End

//////////////////////////////////////
// Select Wavelength of light for monochromator 
// **!! IMPORTANT: **!! 
//This fxn assumes that monochromator gain is given 
// by the relationship below (Till_VoltageIn = ...). Yours may well
// be different, check calibration chart that came w/ your monochromator
//////////////////////////////////////
Function DeliverTillWave(DAC_chan,wavelength) 
	Variable DAC_chan
	Variable wavelength

	If(wavelength>720 || wavelength <320)
		return 0
	EndIf

	Variable Till_VoltageIn = (wavelength-496.6)/21.62
	
	If(Till_voltageIn>=10.23)
		Till_voltageIn = 10.23
	EndIf
	
	String Cmd_voltage = num2str(Till_voltageIn)

	String TillCommand = "ITC18SetDAC " + num2str(DAC_chan) + ", " +Cmd_voltage
	
	Execute TillCommand
End

//////////////////////////////////////
// Panel with wavlength setVar and slider
// Useful for quickly testing excitation spectra, etc. of fluorophores
//////////////////////////////////////
Function AcqMakeTillSelector()
	MakeTillGlobals()
	
	DoWindow/F TillSelector
	If(V_Flag==1)
		return 0
	EndIf
	
	NVAR TillWavelengthChan = root:Packages:TillProcGlobals:TillWavelengthChan
	NVAR/Z IllumChannel = root:Globals:physiology:Run_globals:IlluminationChannel
	
	Variable Active_DAC
	
	If(!NVAR_Exists(IllumChannel))
		Prompt Active_DAC, "Which DAC Channel Controls the Monochromator?",popup, "0;1;2;3"
		DoPrompt  "", Active_DAC
			If(V_flag)
				return 0
			EndIf	
			
		TillWavelengthChan=Active_DAC-1		
	Else
		TillWavelengthChan=IllumChannel
		
	EndIf
	
	PauseUpdate; Silent 1		// building window...
	
	NewPanel/N=TillSelector /W=(263,381,506,480) as "Set Wavelength"
	Slider slider0,pos={8,34},size={225,56},proc=WavelengthSet_slider
	Slider slider0,limits={360,720,1},value= 720,vert= 0
	Slider slider0,userTicks={root:Packages:TillProcGlobals:TillLamp_TickVals,root:Packages:TillProcGlobals:TillLamp_TickLabels}
	SetVariable TillWavelength,pos={63,8},size={113,16},proc=WavelengthSet_setVar,title="Wavelength"
	SetVariable TillWavelength,limits={360,720,0},value= K0
		
	
End


//////////////////////////////////////
// Slider for wavelength control
//////////////////////////////////////
Function WavelengthSet_slider(sa) : SliderControl
	STRUCT WMSliderAction &sa
	NVAR activeDAC = root:Packages:TillProcGlobals:TillWavelengthChan
	switch( sa.eventCode )
		case -1: // kill
			break
		default:
			if( sa.eventCode & 1 ) // value set
				Variable curval = sa.curval
				DeliverTillWave (activeDAC,curval)
			endif
			break
	endswitch

	return 0
End

//////////////////////////////////////
// For Manual setting of wavelength
//////////////////////////////////////
Function WavelengthSet_setVar(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	NVAR DAC_chan = root:Packages:TillProcGlobals:TillWavelengthChan
	

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			Variable dval = sva.dval
			String sval = sva.sval
				DeliverTillWave(DAC_chan,dval)
			break
		case 3: // Live update
		
			
	endswitch

	return 0
End




